#include <stdlib.h>
#include <string.h>
#include <time.h>

#include "radiusd.h"
#include "modules.h"
#include "conffile.h"
#include "rad_assert.h"
#include "rlm_nibs.h"
#include "nibs_utils.h"


static void simply_time_billing( NIBS_USER * nibs_user, REQUEST * request, void *inst)
{
	time_t		tmp_stop_time;
	struct tm	* tim;
	ULONGINT	time_on;
	ULONGINT	seconds_to_hour;
	double		cost;
	char		buf[6];

	tmp_stop_time = request->timestamp;
	time_on = nibs_user->current_time_on - nibs_user->time_on;
	tim = localtime(&tmp_stop_time);
	seconds_to_hour = (ULONGINT)((tim->tm_min * 60) + tim->tm_sec);

	if (seconds_to_hour < time_on)
	{
		snprintf(buf, sizeof(buf),"%02d-%02d", tim->tm_mday, tim->tm_mon+1);
		cost = (nibs_is_holiday(buf, inst)) ? nibs_find_time_cost(nibs_user->gid, 7, tim->tm_hour, inst) : nibs_find_time_cost(nibs_user->gid, tim->tm_wday, tim->tm_hour, inst);
		nibs_user->sub_money += (double)(seconds_to_hour * ((cost + 0.0) / 3600));
		time_on -= seconds_to_hour;
		tmp_stop_time -= (time_t)3600;
		tim = localtime(&tmp_stop_time);

		while (time_on > 3600)
		{
			snprintf(buf, sizeof(buf),"%02d-%02d", tim->tm_mday, tim->tm_mon+1);
			cost = (nibs_is_holiday(buf, inst)) ? nibs_find_time_cost(nibs_user->gid, 7, tim->tm_hour, inst) : nibs_find_time_cost(nibs_user->gid, tim->tm_wday, tim->tm_hour, inst);
			nibs_user->sub_money += cost;
			time_on -= 3600;
			tmp_stop_time -= (time_t)3600;
			tim = localtime(&tmp_stop_time);
		}
	}
	snprintf(buf, sizeof(buf),"%02d-%02d", tim->tm_mday, tim->tm_mon+1);
	cost = (nibs_is_holiday(buf, inst)) ? nibs_find_time_cost(nibs_user->gid, 7, tim->tm_hour, inst) : nibs_find_time_cost(nibs_user->gid, tim->tm_wday, tim->tm_hour, inst);
	nibs_user->sub_money += (double)(time_on * ((cost + 0.0) / 3600));
}

static void simply_traf_billing( NIBS_USER * nibs_user, REQUEST * request, void *instance)
{
	time_t		tmp_stop_time;
	struct tm	* tim;
	int		current_hour;
	double		cost;
	char		buf[6];
	SQL_INST	* inst = (SQL_INST *)instance;

	tmp_stop_time = request->timestamp;
	tim = localtime(&tmp_stop_time);
	current_hour = tim->tm_hour;
	tmp_stop_time = nibs_user->last_change;
	tim = localtime(&tmp_stop_time);

	nibs_user->current_hour_in_bytes = nibs_user->hour_in_bytes + (nibs_user->current_in_bytes - nibs_user->in_bytes);
	nibs_user->current_hour_out_bytes = nibs_user->hour_out_bytes + (nibs_user->current_out_bytes - nibs_user->out_bytes);

	snprintf(buf, sizeof(buf),"%02d-%02d", tim->tm_mday, tim->tm_mon+1);

	switch (nibs_user->direction)
	{
		case 1:
			cost = (nibs_is_holiday(buf, inst)) ? nibs_find_traffic_cost(nibs_user->gid, 7, tim->tm_hour, 1, inst) : nibs_find_traffic_cost(nibs_user->gid, tim->tm_wday, tim->tm_hour, 1, inst);
			nibs_user->current_hour_traffic_money += (((nibs_user->current_hour_in_bytes + 0.0)/inst->config->bytes_per_megabyte) * cost);
		break;
		case 2:
			cost = (nibs_is_holiday(buf, inst)) ? nibs_find_traffic_cost(nibs_user->gid, 7, tim->tm_hour, 2, inst) : nibs_find_traffic_cost(nibs_user->gid, tim->tm_wday, tim->tm_hour, 2, inst);
			nibs_user->current_hour_traffic_money += (((nibs_user->current_hour_out_bytes + 0.0)/inst->config->bytes_per_megabyte) * cost);
		break;
		case 3:
			cost = (nibs_is_holiday(buf, inst)) ? nibs_find_traffic_cost(nibs_user->gid, 7, tim->tm_hour, 1, inst) : nibs_find_traffic_cost(nibs_user->gid, tim->tm_wday, tim->tm_hour, 1, inst);
			nibs_user->current_hour_traffic_money += (((nibs_user->current_hour_in_bytes + 0.0)/inst->config->bytes_per_megabyte) * cost);
			cost = (nibs_is_holiday(buf, inst)) ? nibs_find_traffic_cost(nibs_user->gid, 7, tim->tm_hour, 2, inst) : nibs_find_traffic_cost(nibs_user->gid, tim->tm_wday, tim->tm_hour, 2, inst);
			nibs_user->current_hour_traffic_money += (((nibs_user->current_hour_out_bytes + 0.0)/inst->config->bytes_per_megabyte) * cost);
		break;
		case 4:
			if (nibs_user->current_hour_in_bytes >= nibs_user->current_hour_out_bytes)
			{
				cost = (nibs_is_holiday(buf, inst)) ? nibs_find_traffic_cost(nibs_user->gid, 7, tim->tm_hour, 1, inst) : nibs_find_traffic_cost(nibs_user->gid, tim->tm_wday, tim->tm_hour, 1, inst);
				nibs_user->current_hour_traffic_money += (((nibs_user->current_hour_in_bytes + 0.0)/inst->config->bytes_per_megabyte) * cost);
			}
			else
			{
				cost = (nibs_is_holiday(buf, inst)) ? nibs_find_traffic_cost(nibs_user->gid, 7, tim->tm_hour, 2, inst) : nibs_find_traffic_cost(nibs_user->gid, tim->tm_wday, tim->tm_hour, 2, inst);
				nibs_user->current_hour_traffic_money += (((nibs_user->current_hour_out_bytes + 0.0)/inst->config->bytes_per_megabyte) * cost);
			}
		break;
		case 5:
			if (nibs_user->current_hour_in_bytes <= nibs_user->current_hour_out_bytes)
			{
				cost = (nibs_is_holiday(buf, inst)) ? nibs_find_traffic_cost(nibs_user->gid, 7, tim->tm_hour, 1, inst) : nibs_find_traffic_cost(nibs_user->gid, tim->tm_wday, tim->tm_hour, 1, inst);
				nibs_user->current_hour_traffic_money += (((nibs_user->current_hour_in_bytes + 0.0)/inst->config->bytes_per_megabyte) * cost);
			}
			else
			{
				cost = (nibs_is_holiday(buf, inst)) ? nibs_find_traffic_cost(nibs_user->gid, 7, tim->tm_hour, 2, inst) : nibs_find_traffic_cost(nibs_user->gid, tim->tm_wday, tim->tm_hour, 2, inst);
				nibs_user->current_hour_traffic_money += (((nibs_user->current_hour_out_bytes + 0.0)/inst->config->bytes_per_megabyte) * cost);
			}
		break;
		case 0:
		break;
		default:
			radlog(L_ERR,"rlm_nibs (nibs_billing): Unknown `direction` value for user '%s'",  nibs_user->name);
		break;
	}//direction_switch
	    
	nibs_user->sub_money += nibs_user->current_hour_traffic_money - nibs_user->hour_traffic_money;
	    
	if (current_hour != tim->tm_hour)
	{
		nibs_user->current_hour_in_bytes = 0;
		nibs_user->current_hour_out_bytes = 0;
		nibs_user->current_hour_traffic_money = 0;
	}
}

/*************************************************************************
 *
 *	Function: nibs_billing
 *
 *	Purpose: Return RLM_MODULE_FAIL if error
 *
 *************************************************************************/
int nibs_billing(NIBS_USER *nibs_user, void *instance, REQUEST * request, int acctstatustype)
{
	SQL_INST	*inst = instance;
	SQLSOCK 	*sqlsocket;
	SQL_ROW		row;
	char		querystr[MAX_QUERY_LEN];
	int		num;
	VALUE_PAIR	*pair;

	nibs_user->sub_money = 0;

	DEBUG2("rlm_nibs (nibs_billing): ---== Starting NIBS Billing ==---");

	if ((pair = pairfind(request->packet->vps, PW_ACCT_SESSION_TIME)) != NULL)
	{
		if(pair->lvalue <= 0)
		{
			DEBUG("rlm_nibs (nibs_billing): time_on = 0 (nothing to bill)");
			return RLM_MODULE_OK;
		}
	}
// Added sta
	if (nibs_user->current_in_bytes < nibs_user->in_bytes)
		nibs_user->current_in_bytes = nibs_user->in_bytes;
	if (nibs_user->current_out_bytes < nibs_user->out_bytes)
		nibs_user->current_out_bytes = nibs_user->out_bytes;

	if (nibs_user->current_time_on < nibs_user->time_on)
		nibs_user->current_time_on = nibs_user->time_on;
/* Round to minute if set in config */
	if ( (acctstatustype == PW_STATUS_STOP) && (inst->config->round_to_minute))
	{
		if ( (nibs_user->current_time_on % 60) > 0)
			nibs_user->current_time_on = ((int)(nibs_user->current_time_on/60)+1)*60;
	}
/* Fixed */
	if(nibs_user->fixed > 0)
	{
		radius_xlat(querystr, MAX_QUERY_LEN, inst->config->fixed_select_query, request, als_escape_func);
		if (*querystr)
		{
			sqlsocket = als_get_socket(inst);
			if (sqlsocket == NULL)
				return RLM_MODULE_FAIL;
			query_log(request, inst, querystr);
			if (rlm_nibs_select_query(sqlsocket, inst, querystr))
			{
				snprintf(querystr, sizeof(querystr), "rlm_nibs (nibs_billing): (SQL) Couldn't get fixed statistic for user - %s", (inst->module->als_error)(sqlsocket, inst->config));
				return sql_mod_fail(inst, sqlsocket, querystr);
			}
			if ((inst->module->als_num_rows)(sqlsocket, inst->config) != 0)
			{
				if(rlm_nibs_fetch_row(sqlsocket, inst))
				{
					snprintf(querystr, sizeof(querystr), "rlm_nibs (nibs_billing): SQL query did not succeed for user `%s'", request->username->strvalue);
					return sql_mod_fail( inst, sqlsocket, querystr);
				}
				row = sqlsocket->row;
				num = 0;
				switch (nibs_user->fixed)
				{
					case 1:
						if (Atoi(row[0]) != Atoi(row[1]))
							num = 1;
					break;
					case 2:
						num = (Atoi(row[0]) - Atoi(row[1]));
					break;
					case 3:
						if (acctstatustype == PW_STATUS_START)
							num = 1;
					break;
					case 0:
					case 4:
					break;
					default:
						radlog(L_ERR,"rlm_nibs (nibs_billing): Unknown `fixed` value for user '%s'",  nibs_user->name);
				}
				nibs_user->sub_money += (double)(num * nibs_user->fixed_cost);
			}
			sql_mod_ok( inst, sqlsocket, NULL);
		}
		else
			radlog(L_ERR, "rlm_nibs (nibs_billing): No `fixed_select_query' configured");
		DEBUG2("rlm_nibs (nibs_billing): Money after fixed billing `%14.6f'",  nibs_user->sub_money);
	}
	/* TOS */
	if (nibs_user->tos > 0)
	{
		switch (nibs_user->tos)
		{
			case 1:
				simply_time_billing( nibs_user, request, inst);
			break;
			case 2:
				simply_traf_billing( nibs_user, request, inst);
			break;
			case 3:
				//time part
				simply_time_billing( nibs_user, request, inst);
				//traff part
				simply_traf_billing( nibs_user, request, inst);
				//traff part end
			break;
			default:
				radlog(L_ERR,"rlm_nibs (nibs_billing): Unknown `tos` value for user '%s'",  nibs_user->name);
		}
		DEBUG2("rlm_nibs (nibs_billing): Money after tos billing `%8.6f'",  nibs_user->sub_money);
	}
	nibs_user->sub_total_time = 0;
	if (nibs_user->total_time_limit > 0)
		nibs_user->sub_total_time = (ULONGINT)(nibs_user->current_time_on - nibs_user->time_on);
	else
		nibs_user->sub_total_time -= nibs_user->total_time;
	DEBUG2("rlm_nibs (nibs_billing): sub_total_time `%d'",  (int)nibs_user->sub_total_time);
	nibs_user->sub_total_traffic = 0;
	if (nibs_user->total_traffic_limit > 0)
	{
		switch (nibs_user->direction)
		{
			case 1:
				nibs_user->sub_total_traffic = (ULONGINT)(nibs_user->current_in_bytes - nibs_user->in_bytes);
			break;
			case 2:
				nibs_user->sub_total_traffic = (ULONGINT)(nibs_user->current_out_bytes - nibs_user->out_bytes);
			break;
			case 3:
				nibs_user->sub_total_traffic = (ULONGINT)((nibs_user->current_in_bytes + nibs_user->current_out_bytes) - (nibs_user->in_bytes + nibs_user->out_bytes));
			break;
			case 4:
				nibs_user->sub_total_traffic = (ULONGINT)(nibs_user->current_in_bytes >= nibs_user->current_out_bytes) ? (nibs_user->current_in_bytes - nibs_user->in_bytes) : (nibs_user->current_out_bytes - nibs_user->out_bytes);
			break;
			case 5:
				nibs_user->sub_total_traffic = (ULONGINT)(nibs_user->current_in_bytes <= nibs_user->current_out_bytes) ? (nibs_user->current_in_bytes - nibs_user->in_bytes) : (nibs_user->current_out_bytes - nibs_user->out_bytes);
			break;
			case 0:
			break;
			default:
				radlog(L_ERR,"rlm_nibs (nibs_billing): Unknown `direction' value for user `%s'",  nibs_user->name);
			break;
		}
	}
	else
		nibs_user->sub_total_traffic -= nibs_user->total_traffic;
	DEBUG2("rlm_nibs (nibs_billing): sub_total_traffic `%lu'",  nibs_user->sub_total_traffic);
	nibs_user->sub_total_money = 0;
	if (nibs_user->total_money_limit > 0)
		nibs_user->sub_total_money = (double)(nibs_user->sub_money);
	else
		nibs_user->sub_total_money -= nibs_user->total_money;
	DEBUG2("rlm_nibs (nibs_billing): sub_total_money `%14.6f'",  nibs_user->sub_total_money);
	DEBUG2("rlm_nibs (nibs_billing): ---== NIBS Billing DONE! ==---");
	return RLM_MODULE_OK;
}
