#include <stdlib.h>
#include <string.h>
#include <time.h>

#include "radiusd.h"
#include "modules.h"
#include "conffile.h"
#include "rad_assert.h"
#include "rlm_nibs.h"
#include "nibs_utils.h"

void cleanup_groups( void * instance)
{
        SQL_INST	*inst = instance;
	PACKET		*n_packet, *n_packet_next;

	/* Clear packets */
	for (n_packet = inst->config->first_packet; n_packet; n_packet = n_packet_next)
	{
		n_packet_next = n_packet->next;
		free(n_packet);
	}
	inst->config->first_packet = NULL;
	inst->config->packets = 0;
}

/*************************************************************************
 *
 *  Function: sql_init_groups
 *
 *  Purpose: Initialize groups
 *
 *************************************************************************/
int sql_init_groups(void *instance)
{
	SQLSOCK		*sqlsocket = NULL;
	SQL_INST	*inst = instance;
	SQL_ROW		row;
	char		querystr[MAX_QUERY_LEN];
	PACKET		*n_packet;

	memset(querystr, 0, MAX_QUERY_LEN);

/*Fill groups*/
	if (inst->config->group_select_query)
	{
		sqlsocket = als_get_socket(inst);
		if (sqlsocket == NULL)
			return RLM_MODULE_FAIL;

		query_log(NULL, inst, inst->config->group_select_query);
		if (rlm_nibs_select_query(sqlsocket, inst, inst->config->group_select_query))
		{
			snprintf( querystr, sizeof(querystr), "rlm_nibs (sql_init_groups): Couldn't get set of groups - %s",(inst->module->als_error)(sqlsocket, inst->config));
			return sql_mod_fail( inst, sqlsocket, querystr);
		}
		if ((inst->module->als_num_rows)(sqlsocket, inst->config) == 0)
			return sql_mod_ok(inst, sqlsocket,"rlm_nibs (sql_init_groups): No groups configured");

/*Clear groups*/
		cleanup_groups( inst);
		while (rlm_nibs_fetch_row(sqlsocket, inst) == 0)
		{
			row = sqlsocket->row;
			if (row == NULL)
				break;
			if((n_packet = rad_malloc(sizeof(PACKET))) == NULL)
				return sql_mod_fail(inst, sqlsocket, "rlm_nibs (sql_init_groups): out of memory");;
			memset(n_packet, 0, sizeof(PACKET));
			n_packet->gid = Atoll(row[0]);
			if (row[1] && (row[1][0] != 0)) strNcpy(n_packet->prefix,row[1],sizeof(n_packet->prefix)); else n_packet->prefix[0] = 0;
			n_packet->tos = Atoi(row[2]);
			n_packet->do_with_tos = Atoi(row[3]);
			n_packet->direction = Atoi(row[4]);
			n_packet->fixed = Atoi(row[5]);
			n_packet->fixed_cost = Atof(row[6]);
			n_packet->activation_time = Atoi(row[7]);
			n_packet->total_time_limit = Atoll(row[8]);
			n_packet->month_time_limit = Atoi(row[9]);
			n_packet->week_time_limit = Atoi(row[10]);
			n_packet->day_time_limit = Atoi(row[11]);
			n_packet->total_traffic_limit = Atoll(row[12]);
			n_packet->month_traffic_limit = Atoll(row[13]);
			n_packet->week_traffic_limit = Atoll(row[14]);
			n_packet->day_traffic_limit = Atoll(row[15]);
			n_packet->total_money_limit = Atof(row[16]);
			n_packet->month_money_limit = Atof(row[17]);
			n_packet->week_money_limit = Atof(row[18]);
			n_packet->day_money_limit = Atof(row[19]);
			n_packet->session_timeout = Atoi(row[20]);
			n_packet->idle_timeout = Atoi(row[21]);
			n_packet->simultaneous_use = Atoi(row[22]);
			n_packet->port_limit = Atoi(row[23]);
			if (row[24] && (row[24][0] != 0)) strNcpy(n_packet->login_time,row[24],sizeof(n_packet->login_time)); else n_packet->login_time[0] = 0;
			if (row[25] && (row[25][0] != 0)) strNcpy(n_packet->other_params,row[25],sizeof(n_packet->other_params)); else n_packet->other_params[0] = 0;
			if (row[26] && (row[26][0] != 0)) strNcpy(n_packet->huntgroup_name,row[26],sizeof(n_packet->huntgroup_name)); else n_packet->huntgroup_name[0] = 0;
			n_packet->allow_callback = as_bool((int)row[27][0]);
			if (row[28] && (row[28][0] != 0)) strNcpy(n_packet->allowed_prefixes,row[28],sizeof(n_packet->allowed_prefixes)); else n_packet->allowed_prefixes[0] = 0;
			if (row[29] && (row[29][0] != 0)) strNcpy(n_packet->framed_ip,row[29],sizeof(n_packet->framed_ip)); else n_packet->framed_ip[0] = 0;
			if (row[30] && (row[30][0] != 0)) strNcpy(n_packet->framed_mask,row[30],sizeof(n_packet->framed_mask)); else n_packet->framed_mask[0] = 0;
			n_packet->no_acct = as_bool((int)row[31][0]);
			n_packet->no_pass = as_bool((int)row[32][0]);
			n_packet->next = inst->config->first_packet;
			inst->config->first_packet = n_packet;
			inst->config->packets++;
		}
		sql_mod_ok(inst, sqlsocket, NULL);
	}
	return RLM_MODULE_OK;
}

/*************************************************************************
 *
 *	Function: nibs_packet_find
 *
 *	Purpose: Return number group by name
 *
 *************************************************************************/

PACKET *nibs_packet_find(void *instance, ULONGINT gid)
{
	SQL_INST	*inst = instance;
	PACKET		*pack;

	for (pack = inst->config->first_packet; pack; pack = pack->next)
		if (pack->gid == gid)
			return pack;
	return NULL;
}
