#include <stdlib.h>
#include <string.h>
#include <time.h>

#include "radiusd.h"
#include "modules.h"
#include "conffile.h"
#include "rad_assert.h"
#include "rlm_nibs.h"
#include "nibs_utils.h"

/*************************************************************************
 *
 *  Function: sql_init_holidays
 *
 *  Purpose: Initialize holidays
 *
 *************************************************************************/
int sql_init_holidays(void *instance)
{
	SQLSOCK		*sqlsocket = NULL;
	SQL_INST	*inst = instance;
	SQL_ROW		row;
	char		querystr[MAX_QUERY_LEN];
	HOLIDAY		*holi_day;

	memset(querystr, 0, MAX_QUERY_LEN);
	cleanup_holidays( inst);
/*Fill holidays*/
	if (inst->config->holyday_select_query)
	{
		sqlsocket = als_get_socket(inst);
		if (sqlsocket == NULL)
			return RLM_MODULE_FAIL;
		query_log(NULL, inst, inst->config->holyday_select_query);
		if (rlm_nibs_select_query(sqlsocket, inst, inst->config->holyday_select_query))
		{
			snprintf( querystr, sizeof(querystr), "rlm_nibs (sql_init_holidays): Couldn't get set of holidays - %s",(inst->module->als_error)(sqlsocket, inst->config));
			return sql_mod_fail( inst, sqlsocket, querystr);
		}
		if ((inst->module->als_num_rows)(sqlsocket, inst->config) == 0)
			return sql_mod_ok( inst, sqlsocket, "rlm_nibs (sql_init_holidays): No holidays configured");

		while (rlm_nibs_fetch_row(sqlsocket, inst) == 0)
		{
			row = sqlsocket->row;
			if (!row)
				break;
			if((holi_day = rad_malloc(sizeof(HOLIDAY))) == NULL)
				return sql_mod_fail( inst, sqlsocket, "rlm_nibs (sql_init_holidays): out of memory");
			memset(holi_day, 0, sizeof(HOLIDAY));
			if (row[0]) strNcpy(holi_day->holiday_date,row[0],sizeof(holi_day->holiday_date));
			holi_day->next = inst->config->first_holiday;
			inst->config->first_holiday = holi_day;
			inst->config->holidays++;
		}
		sql_mod_ok( inst, sqlsocket, NULL);
	}
	return RLM_MODULE_OK;
}

void cleanup_holidays( void * instance)
{
        SQL_INST	*inst = instance;
	HOLIDAY		*holi_day, *holi_day_next;

	/*Clear holidays*/
	for (holi_day = inst->config->first_holiday; holi_day; holi_day = holi_day_next)
	{
		holi_day_next = holi_day->next;
		free(holi_day);
	}
	inst->config->first_holiday = NULL;
	inst->config->holidays = 0;
}

/*************************************************************************
 *
 *	Function: nibs_is_holiday
 *
 *	Purpose: Return 1 if today is holiday
 *
 *************************************************************************/
int nibs_is_holiday(char *dbuf, void *instance)
{
	SQL_INST	*inst = instance;
	HOLIDAY		*first;
	char		buf[6];
	struct tm	*tim;

	if (dbuf == NULL)
	{
		tim = localtime(NULL);
		strftime(buf, sizeof(buf), "%d-%m", tim);
	}
	else
		strNcpy(buf, dbuf, sizeof(buf));

	for (first = inst->config->first_holiday; first; first = first->next)
		if (strncmp(first->holiday_date, buf, sizeof(first->holiday_date)) == 0)
		{
			DEBUG2("rlm_nibs (nibs_is_holiday): Day '%s' is holiday",  buf);
			return 1;
		}
	return 0;
}
