#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/file.h>
#include <fcntl.h>
#include <ctype.h>
#include <signal.h>

#if HAVE_SYS_WAIT_H
#	include <sys/wait.h>
#endif
#ifndef WEXITSTATUS
#	define WEXITSTATUS(stat_val) ((unsigned)(stat_val) >> 8)
#endif
#ifndef WIFEXITED
#	define WIFEXITED(stat_val) (((stat_val) & 255) == 0)
#endif

#include "radiusd.h"
#include "modules.h"
#include "conffile.h"
#include "rad_assert.h"
#include "rlm_nibs.h"
#include "nibs_utils.h"

/*************************************************************************
 *
 *	Function: nibs_user_kill
 *
 *	Purpose: kill user
 *
 *************************************************************************/
int nibs_user_kill(void *instance, REQUEST * request)
{
	SQL_INST	*inst = instance;
	VALUE_PAIR *vp;
	char answer[4096];
	char *argv[256];
	char *buf, *p;
	pid_t pid;
	int argc = -1;
	int n;

	/*
	 *	Do the translation (as the parent) of the command to
	 *	execute.  This MAY involve calling other modules, so
	 *	we want to do it in the parent.
         */
	radius_xlat(answer, sizeof(answer), inst->config->kill_program, request, NULL);
        buf = answer;

	/*
	 *	Log the command if we are debugging something
	 */
	DEBUG("kill_program: %s", buf);

	/*
	 *	Build vector list of arguments and execute.
	 *
	 *	FIXME: This parsing gets excited over spaces in
	 *	the translated strings, e.g. User-Name = "aa bb"
	 *	is passed as two seperate arguments, instead of one.
	 *
	 *	What we SHOULD do instead is to split the exec program
	 *	buffer first, and then do the translation on every
	 *	subsequent string.
	 */
	p = strtok(buf, " \t");
	if (p) do
	{
		argv[++argc] = p;
		p = strtok(NULL, " \t");
	} while(p != NULL);

	argv[++argc] = p;
	if (argc == 0)
	{
		radlog(L_ERR, "Exec-Program: empty command line.");
		return -1;
	}

	if ((pid = rad_fork(0)) == 0)
	{
#define MAX_ENVP 1024
		int i, devnull;
		char *envp[MAX_ENVP];
		int envlen;
		char buffer[1024];

		/*	
		 *	Child process.
		 *
		 *	We try to be fail-safe here.  So if ANYTHING
		 *	goes wrong, we exit with status 1.
		 */

		/*
		 *	Open STDIN to /dev/null
		 */
		devnull = open("/dev/null", O_RDWR);
		if (devnull < 0)
		{
			radlog(L_ERR|L_CONS, "Failed opening /dev/null: %s\n",
			       strerror(errno));
			exit(1);
		}
		dup2(devnull, STDIN_FILENO);
		dup2(devnull, STDOUT_FILENO);

		/*
		 *	If we're not debugging, then we can't do
		 *	anything with the error messages, so we throw
		 *	them away.
		 *
		 *	If we are debugging, then we want the error
		 *	messages to go to the STDERR of the server.
		 */
		if (debug_flag == 0)
			dup2(devnull, STDERR_FILENO);
		close(devnull);

		/*
		 *	The server may have MANY FD's open.  We don't
		 *	want to leave dangling FD's for the child process
		 *	to play funky games with, so we close them.
		 */
		for (i = 3; i < 256; i++)
			close(i);
		/*
		 *	Set up the environment variables.
		 *	We're in the child, and it will exit in 4 lines
		 *	anyhow, so memory allocation isn't an issue.
		 */
		envlen = 0;

		for (vp = request->packet->vps; vp->next; vp = vp->next)
		{
			/*
			 *	Hmm... maybe we shouldn't pass the
			 *	user's password in an environment
			 *	variable...
			 */
			snprintf(buffer, sizeof(buffer), "%s=", vp->name);
			for (p = buffer; *p != '='; p++)
			{
				if (*p == '-')
				{
					*p = '_';
				}
				else
					if (isalpha((int) *p))
					{
						*p = toupper(*p);
					}
			}

			n = strlen(buffer);
			vp_prints_value(buffer+n, sizeof(buffer) - n, vp, 1);

			envp[envlen++] = strdup(buffer);
		}
		envp[envlen] = NULL;

		execve(argv[0], argv, envp);

		radlog(L_ERR, "kill_program: FAILED to execute %s: %s",
		       argv[0], strerror(errno));
		exit(1);
	}

	/*
	 *	Parent process.
	 */
	if (pid < 0)
	{
		radlog(L_ERR|L_CONS, "Couldn't fork %s: %s",
		       argv[0], strerror(errno));
		return -1;
	}

	return 0;
}

/*************************************************************************
 *
 *	Function: nibs_add_current
 *
 *	Purpose: Add current counters status
 *
 *************************************************************************/
void nibs_add_current(NIBS_USER *nibs_user, REQUEST * request)
{
	VALUE_PAIR	*pair;
	char		logstr[MAX_QUERY_LEN];
	char		logstr2[MAX_QUERY_LEN];
	char		*call_from;
	int		acctstatustype = 0;

	nibs_user->current_time_on = 0;
	nibs_user->current_in_bytes = 0;
	nibs_user->current_out_bytes = 0;
	if ((pair = pairfind(request->packet->vps, PW_ACCT_STATUS_TYPE)) != NULL)
		acctstatustype = pair->lvalue;

	if ((pair = pairfind(request->packet->vps, PW_CALLING_STATION_ID)) != NULL)
		call_from = pair->strvalue;
	else
		call_from = NULL;
	if ((pair = pairfind(request->packet->vps, PW_ACCT_SESSION_TIME)) != NULL)
	{
		if(pair->lvalue <= 0)
		{
			nibs_user->current_time_on = nibs_user->time_on;
			if (acctstatustype == PW_STATUS_ALIVE)
				radius_xlat(logstr2, sizeof(logstr2), "rlm_nibs (nibs_add_current): Alive packet with zero session length for user `%{SQL-User-Name}' [%{NAS-IP-Address}:%{NAS-Port}]%%s%%s%%s", request, als_escape_func);
			if (acctstatustype == PW_STATUS_STOP)
				radius_xlat(logstr2, sizeof(logstr2), "rlm_nibs (nibs_add_current): Stop packet with zero session length for user `%{SQL-User-Name}' [%{NAS-IP-Address}:%{NAS-Port}]%%s%%s%%s", request, als_escape_func);
			snprintf(logstr, sizeof(logstr), logstr2, (call_from) ? " |" : "", (call_from) ? call_from : "", (call_from) ? "|" : "");
			radlog(L_ERR, logstr);
		}
		else
			nibs_user->current_time_on = (ULONGINT)pair->lvalue;
	}
	if ((pair = pairfind(request->packet->vps, PW_ACCT_INPUT_OCTETS)) != NULL)
		nibs_user->current_in_bytes = (ULONGINT)pair->lvalue;
	if ((pair = pairfind(request->packet->vps, PW_ACCT_OUTPUT_OCTETS)) != NULL)
		nibs_user->current_out_bytes = (ULONGINT)pair->lvalue;
}

/*************************************************************************
 *
 *	Function: nibs_add_attrs
 *
 *	Purpose: Return RLM_MODULE_OK if all done
 *
 *************************************************************************/
int nibs_add_attrs(NIBS_USER *nibs_user, void *instance, REQUEST * request)
{
	SQL_INST	*inst = instance;
	LRAD_TOKEN	parsecode;
	VALUE_PAIR	*pair;
	int		nibs_session_timeout;
	char		tmpbuf[100];

	if (nibs_user == NULL)
		return RLM_MODULE_FAIL;

	DEBUG2("rlm_nibs (nibs_add_attrs): begin for user `%s' ------------", request->username->strvalue);

//passwd
	if ((nibs_user->passwd != 0) && (nibs_user->crypt_method == 0))
	{
		DEBUG2("rlm_nibs (nibs_add_attrs): add PW_PASSWORD");
		if ((pair = pairfind(request->config_items, PW_PASSWORD)) != NULL)
			pairdelete(&request->config_items, PW_PASSWORD);
		pair = pairmake("User-Password", nibs_user->passwd, T_OP_CMP_EQ);
		if (pair == NULL) 
			radlog(L_ERR, "%s", librad_errstr);
		else
			pairadd(&request->config_items, pair);
	}

//framed ip
	if (nibs_user->framed_ip[0] != 0)
	{
		DEBUG2("rlm_nibs (nibs_add_attrs): add PW_FRAMED_IP_ADDRESS");
		if ((pair = pairfind(request->reply->vps, PW_FRAMED_IP_ADDRESS)) != NULL)
			pairdelete(&request->reply->vps, PW_FRAMED_IP_ADDRESS);
		pair = pairmake("Framed-IP-Address", nibs_user->framed_ip, T_OP_EQ);
		if (pair == NULL)
			radlog(L_ERR, "%s", librad_errstr);
		else
			pairadd(&request->reply->vps, pair);
	}
    
//framed mask
	if (nibs_user->framed_mask[0] != 0)
	{
		DEBUG2("rlm_nibs (nibs_add_attrs): add PW_FRAMED_IP_NETMASK");
		if ((pair = pairfind(request->reply->vps, PW_FRAMED_IP_NETMASK)) != NULL)
			pairdelete(&request->reply->vps, PW_FRAMED_IP_NETMASK);
		pair = pairmake("Framed-IP-Netmask", nibs_user->framed_mask, T_OP_EQ);
		if (pair == NULL)
			radlog(L_ERR, "%s", librad_errstr);
		else
			pairadd(&request->reply->vps, pair);
	}
    
//callback number
	if (nibs_user->allow_callback)
	{
		if ((pair = pairfind(request->reply->vps, PW_CALLBACK_NUMBER)) != NULL)
		    pairdelete(&request->reply->vps, PW_CALLBACK_NUMBER);
		switch ( get_nas_type(nibs_user->nas_ip))
		{
		    case NAS_ASCEND:
		    
			bzero(tmpbuf,100);
			if ((pair = pairfind(request->packet->vps, PW_HINT)) != NULL)
			    strncpy(tmpbuf, pair->strvalue, 100);
			DEBUG2("rlm_nibs (nibs_add_attrs): HINT=%s",tmpbuf);
			if (strcmp(tmpbuf,"CB") == 0)
			{
			    if ( nibs_user->callback_number && (strlen(nibs_user->callback_number) > 0))
			    {
				DEBUG2("rlm_nibs (nibs_add_attrs): add PW_CALLBACK_NUMBER");
				pair = pairmake("Ascend-Dial-Number", nibs_user->callback_number, T_OP_EQ);
				if (pair == NULL)
				    radlog(L_ERR, "%s", librad_errstr);
				else 
				    pairadd(&request->reply->vps, pair);
				pair = pairmake("Ascend-CBCP-Mode", "CBCP-Profile-Callback", T_OP_EQ);
				if (pair == NULL)
				    radlog(L_ERR, "%s", librad_errstr);
				else
				    pairadd(&request->reply->vps, pair);
			    }
			    else
			    {
				pair = pairmake("Ascend-CBCP-Mode", "CBCP-User-Callback", T_OP_EQ);
				if (pair == NULL)
				    radlog(L_ERR, "%s", librad_errstr);
				else
				    pairadd(&request->reply->vps, pair);
			    }
			    pair = pairmake("Ascend-CBCP-Enable", "CBCP-Enabled", T_OP_EQ);
			}
			else
			    pair = pairmake("Reply-Message", "CallBack for you has been disabled.", T_OP_EQ);
		    break;
		    case NAS_CISCO:
			bzero( tmpbuf, 100);
			snprintf( tmpbuf, 100, "lcp:callback-dialstring=%s", nibs_user->callback_number);
			pair = pairmake("Cicso-AVPair", tmpbuf, T_OP_EQ);
		    break;
		    default:
			DEBUG2("rlm_nibs (nibs_add_attrs): add PW_CALLBACK_NUMBER");
			pair = pairmake("Callback-Number", nibs_user->callback_number, T_OP_EQ);
		    break;
		}
		if (pair == NULL)
		    radlog(L_ERR, "%s", librad_errstr);
		else
		    pairadd(&request->reply->vps, pair);
	}

//idle timeout
	if (nibs_user->idle_timeout > 0)
	{
		DEBUG2("rlm_nibs (nibs_add_attrs): add PW_IDLE_TIMEOUT");
		if ((pair = pairfind(request->reply->vps, PW_IDLE_TIMEOUT)) != NULL)
			pairdelete(&request->reply->vps, PW_IDLE_TIMEOUT);
		if ((pair = paircreate(PW_IDLE_TIMEOUT, PW_TYPE_INTEGER)) == NULL)
			radlog(L_ERR, "%s", librad_errstr);
		else
		{
			pair->lvalue = nibs_user->idle_timeout;
			pair->operator = T_OP_EQ;
			pairadd(&request->reply->vps, pair);
		}
	}

//simultaneous use
	if (nibs_user->simultaneous_use > 0)
	{
		DEBUG2("rlm_nibs (nibs_add_attrs): add PW_SIMULTANEOUS_USE");
		if ((pair = pairfind(request->config_items, PW_SIMULTANEOUS_USE)) != NULL)
			pairdelete(&request->config_items, PW_SIMULTANEOUS_USE);
		if ((pair = paircreate(PW_SIMULTANEOUS_USE, PW_TYPE_INTEGER)) == NULL)
			radlog(L_ERR, "%s", librad_errstr);
		else
		{
			pair->lvalue = nibs_user->simultaneous_use;
			pair->operator = T_OP_SET;
			pairadd(&request->config_items, pair);
		}
	}

//Huntgroup
	if (nibs_user->huntgroup_name[0] != 0)
	{
		DEBUG2("rlm_nibs (nibs_add_attrs): add PW_HUNTGROUP_NAME");
		if ((pair = pairfind(request->config_items, PW_HUNTGROUP_NAME)) != NULL)
			pairdelete(&request->config_items, PW_HUNTGROUP_NAME);
		pair = pairmake("Huntgroup-Name", nibs_user->huntgroup_name, T_OP_CMP_EQ);
		if (pair == NULL)
			radlog(L_ERR, "%s", librad_errstr);
		else
			pairadd(&request->config_items, pair);
	}

//session timeout
	DEBUG2("rlm_nibs (nibs_add_attrs): add PW_SESSION_TIMEOUT");
	if ( (nibs_user->session_timeout > 31536000) || (nibs_user->session_timeout == 0))
	    nibs_user->session_timeout = 31536000;
	if (inst->config->set_session_timeout)
	{
		nibs_session_timeout = nibs_set_session_timeout(nibs_user, inst, request->timestamp);
		if ((nibs_user->session_timeout == 0) || (nibs_user->session_timeout > nibs_session_timeout))
			nibs_user->session_timeout = nibs_session_timeout;
	}
	if (nibs_user->session_timeout > 0)
	{
		if ((pair = pairfind(request->reply->vps, PW_SESSION_TIMEOUT)) != NULL)
			pairdelete(&request->reply->vps, PW_SESSION_TIMEOUT);
		if ((pair = paircreate(PW_SESSION_TIMEOUT, PW_TYPE_INTEGER)) == NULL)
			radlog(L_ERR, "%s", librad_errstr);
		else
		{
			pair->operator = T_OP_EQ;
			pair->lvalue = nibs_user->session_timeout;
			pairadd(&request->reply->vps, pair);
		}
	}
// Session octets
	if ( inst->config->set_max_octet_limit)
	{
		if ((pair = pairfind(request->reply->vps, 227)) != NULL)
			pairdelete(&request->reply->vps, 227);
		if ((pair = paircreate(227, PW_TYPE_INTEGER)) == NULL)
			radlog(L_ERR, "%s", librad_errstr);
		else
		{
			pair->operator = T_OP_EQ;
			pair->lvalue = inst->config->max_octet_limit;
			pairadd(&request->reply->vps, pair);
		}
		if ((pair = pairfind(request->reply->vps, 228)) != NULL)
			pairdelete(&request->reply->vps, 228);
		if ((pair = paircreate(228, PW_TYPE_INTEGER)) == NULL)
			radlog(L_ERR, "%s", librad_errstr);
		else
		{
			pair->operator = T_OP_EQ;
			pair->lvalue = 4;
			pairadd(&request->reply->vps, pair);
		}
	}
//port limit
	if (nibs_user->port_limit > 0)
	{
		DEBUG2("rlm_nibs (nibs_add_attrs): add PW_PORT_LIMIT");
		if ((pair = pairfind(request->reply->vps, PW_PORT_LIMIT)) != NULL)
			pairdelete(&request->reply->vps, PW_PORT_LIMIT);
		if ((pair = paircreate(PW_PORT_LIMIT, PW_TYPE_INTEGER)) == NULL)
			radlog(L_ERR, "%s", librad_errstr);
		else
		{
			pair->operator = T_OP_EQ;
			pair->lvalue = nibs_user->port_limit;
			pairadd(&request->reply->vps, pair);
		}
	}

//other_params
	if (nibs_user->other_params[0] != 0)
	{
		DEBUG2("rlm_nibs (nibs_add_attrs): add all other params");
		/*
		*  Replace any trailing comma by a NUL.
		*/
		if (nibs_user->other_params[strlen(nibs_user->other_params) - 1] == ',')
			nibs_user->other_params[strlen(nibs_user->other_params) - 1] = '\0';
		pair = NULL;
		parsecode = userparse(nibs_user->other_params, &pair);
		if (parsecode < 1)
			radlog(L_ERR|L_CONS, "rlm_nibs (nibs_add_attrs): Parse error (reply) at SQL other_params");
		else
		{
			pairxlatmove(request, &request->reply->vps, &pair);
			pairfree(&pair);
		}
	}

	DEBUG2("rlm_nibs (nibs_add_attrs): end for user `%s' ------------", request->username->strvalue);
	return RLM_MODULE_OK;
}

static void calc_st( NIBS_USER * nibs_user, void * instance, double * d, time_t tso, int * st)
{
	char		buf[6];
	int		sec_to_hour;
	double		cost, deposit = * d;
	int		secs = 0;
	struct tm	*tim;
	time_t		ts;
	SQL_INST	* inst = instance;

	if (((nibs_user->tos == 1) || (nibs_user->tos == 3)) && (deposit > 0))
	{
		ts = tso;
		tim = localtime(&ts);
		sec_to_hour = (((59 - tim->tm_min) * 60) + (60 - tim->tm_sec));
		snprintf(buf, sizeof(buf), "%02d-%02d", tim->tm_mday, tim->tm_mon + 1);
		cost = (nibs_is_holiday(buf, inst)) ? (nibs_find_time_cost(nibs_user->gid, 7, tim->tm_hour, inst) / 3600) : (nibs_find_time_cost(nibs_user->gid, tim->tm_wday, tim->tm_hour, inst) / 3600);
		if ((sec_to_hour * cost) < deposit)
		{
			// more then to hour end
			deposit -= (sec_to_hour * cost);
			secs = sec_to_hour;
			while (1)
			{
				ts += 3600;
				tim = localtime(&ts);
				snprintf(buf, sizeof(buf), "%02d-%02d", tim->tm_mday, tim->tm_mon + 1);
				cost = (nibs_is_holiday(buf, inst)) ? nibs_find_time_cost(nibs_user->gid, 7, tim->tm_hour, inst) : nibs_find_time_cost(nibs_user->gid, tim->tm_wday, tim->tm_hour, inst);
				if (deposit < cost)
					break;
				if (secs >= MAX_EXPIRE)
				{
					deposit = 0;
					break;
				}
				deposit -= cost;
				secs += 3600;
			}
			if (cost)
				secs += Round(deposit / (cost / 3600));
		}
		else
		{
			// less then to hour end
			if (cost)
				deposit = (deposit / cost);
			if ((deposit > 0) && (deposit < 1))
				deposit = 1;
			secs = Round(deposit);
		}
	if ((*st == 0) || ((secs < *st) && (secs > 0)))
		*st = secs;
	}
}

/*************************************************************************
 *
 *	Function: nibs_set_session_timeout
 *
 *	Purpose: Return calculated timeout
 *
 *************************************************************************/
int nibs_set_session_timeout(NIBS_USER *nibs_user, void *instance, time_t tso)
{
	SQL_INST	*inst = instance;
	int		session_tout, tout_tmp;
	double		deposit;

	if (nibs_user == NULL)
		return -1;
	session_tout = 0;
	//Login Time
	if (nibs_user->login_time[0] != 0)
	{
		tout_tmp = Timestr_match(nibs_user->login_time, tso, inst);
		if ((session_tout == 0) || ((tout_tmp < session_tout) && (tout_tmp > 0)))
			session_tout = tout_tmp;
	}
	// Expired
	if ((nibs_user->expired != 0) && (nibs_user->expired > (int) tso))
	{
		tout_tmp = (nibs_user->expired - (int) tso);
		if ((session_tout == 0) || ((tout_tmp < session_tout) && (tout_tmp > 0)))
			session_tout = tout_tmp;
	}

	// Hour Total Limit
	if ((nibs_user->total_time_limit > 0) && (nibs_user->total_time_limit > nibs_user->total_time))
	{
		tout_tmp = (nibs_user->total_time_limit - nibs_user->total_time);
		if ((session_tout == 0) || ((tout_tmp < session_tout) && (tout_tmp > 0)))
			session_tout = tout_tmp;
	}
	// Hour Month Limit
	if ((nibs_user->month_time_limit > 0) && (nibs_user->month_time_limit > nibs_user->month_time))
	{
		tout_tmp = (nibs_user->month_time_limit - nibs_user->month_time);
		if ((session_tout == 0) || ((tout_tmp < session_tout) && (tout_tmp > 0)))
			session_tout = tout_tmp;
	}
	// Hour Week Limit
	if ((nibs_user->week_time_limit > 0) && (nibs_user->week_time_limit > nibs_user->week_time))
	{
		tout_tmp = (nibs_user->week_time_limit - nibs_user->week_time);
		if ((session_tout == 0) || ((tout_tmp < session_tout) && (tout_tmp > 0)))
			session_tout = tout_tmp;
	}
	// Hour Day Limit
	if ((nibs_user->week_time_limit > 0) && (nibs_user->week_time_limit > nibs_user->week_time))
	{
		tout_tmp = (nibs_user->week_time_limit - nibs_user->week_time);
		if ((session_tout == 0) || ((tout_tmp < session_tout) && (tout_tmp > 0)))
			session_tout = tout_tmp;
	}

	// Money-Time
	if (nibs_user->do_with_tos == 1)
	{
		deposit = nibs_user->deposit + nibs_user->credit;
		calc_st( nibs_user, instance, &deposit, tso, &session_tout);
	}

// Money-Limits
	// Money-Total-Limit
	if (nibs_user->total_money_limit > 0)
	{
		deposit = nibs_user->total_money_limit - nibs_user->total_money;
		calc_st( nibs_user, instance, &deposit, tso, &session_tout);
	}
	// Money-Month-Limit
	if (nibs_user->month_money_limit > 0)
	{
		deposit = nibs_user->month_money_limit - nibs_user->month_money;
		calc_st( nibs_user, instance, &deposit, tso, &session_tout);
	}
	// Money-Week-Limit
	if (nibs_user->week_money_limit > 0)
	{
		deposit = nibs_user->week_money_limit - nibs_user->week_money;
		calc_st( nibs_user, instance, &deposit, tso, &session_tout);
	}
	// Money-Day-Limit
	if (nibs_user->day_money_limit > 0)
	{
		deposit = nibs_user->day_money_limit - nibs_user->day_money;
		calc_st( nibs_user, instance, &deposit, tso, &session_tout);
	}
	return session_tout;
}


/*************************************************************************
 *
 *	Function: nibs_filter
 *
 *	Purpose: Return !0 if user must not connect
 *
 *************************************************************************/
int nibs_filter(NIBS_USER *nibs_user, void *instance, REQUEST * request)
{
	SQL_INST	*inst = instance;
    	SQLSOCK 	*sqlsocket;
	SQL_ROW		row;
	char		querystr[MAX_QUERY_LEN];
	char		querystr2[MAX_QUERY_LEN];
	char		buf1[32];
	char		buf2[32];
//	char		*tmp_str;
	unsigned long int tmpll;
	struct tm	*tim;
	time_t		curtime, tmptime;

	if (nibs_user == NULL)
	    return -1;

	curtime = tmptime = time(NULL);
	tmpll = 0;

	DEBUG2("rlm_nibs (nibs_filter): begin for user '%s' ------------",  request->username->strvalue);

//Money
	if (nibs_user->do_with_tos == 1)
	{
		if ((nibs_user->tos > 0) || (nibs_user->fixed > 0))
		{
			DEBUG2("rlm_nibs (nibs_filter): Money filter for user '%s'",  request->username->strvalue);
			if ((nibs_user->deposit + nibs_user->credit) <= 0)
    				return 1;
		}
	}
//Money limits	
//Total money limit
    	if (nibs_user->total_money_limit > 0)
	{
		DEBUG2("rlm_nibs (nibs_filter): Total money limit filter for user '%s'",  request->username->strvalue);
		if (nibs_user->total_money_limit <= nibs_user->total_money)
	    	    return 61;
	}

//Month money limit
    	if (nibs_user->month_money_limit > 0)
	{
		DEBUG2("rlm_nibs (nibs_filter): Month money limit filter for user '%s'",  request->username->strvalue);
	    	radius_xlat(querystr, sizeof(querystr), inst->config->month_money_query, request, als_escape_func);

		if(*querystr)
		{
			sqlsocket = als_get_socket(inst);
			if (sqlsocket == NULL)
				return -1;

			query_log(request, inst, querystr);
    			if (rlm_nibs_select_query(sqlsocket, inst, querystr))
			{
        			radlog(L_ERR, "rlm_nibs (nibs_filter): Couldn't get sum of month money - database query error `%s'", (inst->module->als_error)(sqlsocket, inst->config));
				return -1;
			}
			if(rlm_nibs_fetch_row(sqlsocket, inst))
			{
				radlog(L_ERR, "rlm_nibs (nibs_filter): Couldn't get sum of month money - SQL query did not succeed for user `%s'", request->username->strvalue);
				sql_mod_ok( inst, sqlsocket, NULL);
				return -1;
			}
			row = sqlsocket->row;
			nibs_user->month_money = Atoi(row[0]);
			sql_mod_ok( inst, sqlsocket, NULL);
			if (nibs_user->month_money_limit <= nibs_user->month_money)
				return 62;
			else
				radlog(L_ERR,"rlm_nibs (nibs_filter): No `month_money_query' configured");
		}

//Week money limit
		if (nibs_user->week_money_limit > 0)
		{
			DEBUG2("rlm_nibs (nibs_filter): Week money limit filter for user '%s'",  request->username->strvalue);
		    	radius_xlat(querystr, sizeof(querystr), inst->config->week_money_query, request, als_escape_func);
			if(*querystr)
			{
				sqlsocket = als_get_socket(inst);
				if (sqlsocket == NULL)
					return -1;
				query_log(request, inst, querystr);
				if (rlm_nibs_select_query(sqlsocket, inst, querystr))
				{
					radlog(L_ERR, "rlm_nibs (nibs_filter): Couldn't get sum of week money - database query error `%s'", (inst->module->als_error)(sqlsocket, inst->config));
					return -1;
				}
				if(rlm_nibs_fetch_row(sqlsocket, inst))
				{
					radlog(L_ERR, "rlm_nibs (nibs_filter): Couldn't get sum of week money - SQL query did not succeed for user `%s'", request->username->strvalue);
					sql_mod_ok( inst, sqlsocket, NULL);
					return -1;
				}
                                row = sqlsocket->row;
                                nibs_user->week_money = Atoi(row[0]);
                                sql_mod_ok( inst, sqlsocket, NULL);
				if (nibs_user->week_money_limit <= nibs_user->week_money)
					return 63;
			}
			else
				radlog(L_ERR,"rlm_nibs (nibs_filter): No `week_money_query' configured");
		}
//Day money limit
		if (nibs_user->day_money_limit > 0)
		{
			DEBUG2("rlm_nibs (nibs_filter): Day money limit filter for user '%s'",  request->username->strvalue);
			radius_xlat(querystr, sizeof(querystr), inst->config->day_money_query, request, als_escape_func);
			if(*querystr)
			{
				sqlsocket = als_get_socket(inst);
				if (sqlsocket == NULL)
					return -1;
				query_log(request, inst, querystr);
				if (rlm_nibs_select_query(sqlsocket, inst, querystr))
				{
					radlog(L_ERR, "rlm_nibs (nibs_filter): Couldn't get sum of day money - database query error `%s'", (inst->module->als_error)(sqlsocket, inst->config));
					return -1;
				}
				if(rlm_nibs_fetch_row(sqlsocket, inst))
				{
					radlog(L_ERR, "rlm_nibs (nibs_filter): Couldn't get sum of day money - SQL query did not succeed for user `%s'", request->username->strvalue);
					sql_mod_ok( inst, sqlsocket, NULL);
					return -1;
				}
				row = sqlsocket->row;
				nibs_user->day_money = Atoi(row[0]);
				sql_mod_ok( inst, sqlsocket, NULL);
				if (nibs_user->day_money_limit <= nibs_user->day_money)
					return 64;
			}
			else
				radlog(L_ERR,"rlm_nibs (nibs_filter): No `day_money_query' configured");
		}
	}
//Activation
	if (nibs_user->activated == 0)
	{
		DEBUG2("rlm_nibs (nibs_filter): Activation filter for user '%s'",  request->username->strvalue);
	        nibs_user->activated = 1;

		if (nibs_user->add_date != 0)
		{
			if ((int)curtime < nibs_user->add_date)
				return 31;
		}
	    	radius_xlat(querystr, sizeof(querystr), inst->config->activation_update_query, request, als_escape_func);
		if(*querystr)
		{
			if (nibs_user->add_date == 0)
			{
				snprintf(buf1, sizeof(buf1), "from_unixtime(%d)", (int)request->timestamp);
				DEBUG2("rlm_nibs (nibs_filter): User '%s': setting add_date to now",  request->username->strvalue);
			}
			else
				snprintf(buf1, sizeof(buf1), "from_unixtime(%d)", nibs_user->add_date);
			if (nibs_user->activation_time == 0)
			{
				snprintf(buf2, sizeof(buf2), "'0'");
				DEBUG2("rlm_nibs (nibs_filter): User '%s' unsetting expired",  request->username->strvalue);
			}
			else
			{
	    			tmptime += (time_t) nibs_user->activation_time;
				tim = localtime(&tmptime);
        			if (nibs_user->expired == 0)
				{
					strftime(buf2, sizeof(buf2), "'%Y-%m-%d'", tim);
					DEBUG2("rlm_nibs (nibs_filter): User '%s': setting expired to %s",  request->username->strvalue, buf2);
				}
				else
				{
					if (tmptime < nibs_user->expired)
					{
						strftime(buf2, sizeof(buf2), "'%Y-%m-%d'", tim);
						DEBUG2("rlm_nibs (nibs_filter): User '%s': change expired to '%s'",  request->username->strvalue , buf2);
					}
					else
					{
						snprintf(buf2, sizeof(buf2), "from_unixtime(%d)", nibs_user->expired);
						DEBUG2("rlm_nibs (nibs_filter): User '%s': leaving expired unchanged",  request->username->strvalue);
					}
				}
			}
			snprintf(querystr2, sizeof(querystr2), querystr, buf1, buf2, nibs_user->activated);
			sqlsocket = als_get_socket(inst);
			if (sqlsocket == NULL)
				return -1;
			query_log(request, inst, querystr2);
			if(rlm_nibs_query(sqlsocket,inst,querystr2))
			{
				radlog(L_ERR,"rlm_nibs (nibs_filter): Couldn't update account for user %s - %s",  request->username->strvalue, (inst->module->als_error)(sqlsocket, inst->config));
				return -1;
			}
			sql_mod_ok( inst, sqlsocket, NULL);
		}
		else
			radlog(L_ERR,"rlm_nibs (nibs_filter): No `activation_update_query' configured");
	}
//Expire
	if (nibs_user->expired != 0)
	{
		DEBUG2("rlm_nibs (nibs_filter): Expire filter for user '%s'",  request->username->strvalue);
		if ((int)curtime >= nibs_user->expired)
			return 2;
	}
//Block
	if (nibs_user->blocked > 0)
	{
		DEBUG2("rlm_nibs (nibs_filter): Block filter for user '%s'",  request->username->strvalue);
		return 3;
	}
//Total time limit
	if (nibs_user->total_time_limit != 0)
	{
		DEBUG2("rlm_nibs (nibs_filter): Total time limit filter for user '%s'",  request->username->strvalue);
		if (nibs_user->total_time_limit <= nibs_user->total_time)
			return 11;
	}
//Month time limit
	if (nibs_user->month_time_limit != 0)
	{
		DEBUG2("rlm_nibs (nibs_filter): Month time limit filter for user '%s'",  request->username->strvalue);
		radius_xlat(querystr, sizeof(querystr), inst->config->month_time_query, request, als_escape_func);
		if(*querystr)
		{
			sqlsocket = als_get_socket(inst);
			if (sqlsocket == NULL)
				return -1;
			query_log(request, inst, querystr);
			if (rlm_nibs_select_query(sqlsocket, inst, querystr))
			{
				radlog(L_ERR, "rlm_nibs (nibs_filter): Couldn't get sum of month time - database query error `%s'", (inst->module->als_error)(sqlsocket, inst->config));
				return -1;
			}
			if(rlm_nibs_fetch_row(sqlsocket, inst))
			{
				radlog(L_ERR, "rlm_nibs (nibs_filter): Couldn't get sum of month time - SQL query did not succeed for user `%s'", request->username->strvalue);
				sql_mod_ok( inst, sqlsocket, NULL);
				return -1;
			}
			row = sqlsocket->row;
			nibs_user->month_time = Atoi(row[0]);
			sql_mod_ok( inst, sqlsocket, NULL);
			if (nibs_user->month_time_limit <= nibs_user->month_time)
				return 12;
		}
		else
			radlog(L_ERR,"rlm_nibs (nibs_filter): No month_time query configured");
	}
//Week time limit
	if (nibs_user->week_time_limit != 0)
	{
		DEBUG2("rlm_nibs (nibs_filter): Week time limit filter for user '%s'",  request->username->strvalue);
		radius_xlat(querystr, sizeof(querystr), inst->config->week_time_query, request, als_escape_func);
		if(*querystr)
		{
			sqlsocket = als_get_socket(inst);
			if (sqlsocket == NULL)
				return -1;
			query_log(request, inst, querystr);
			if (rlm_nibs_select_query(sqlsocket, inst, querystr))
			{
				radlog(L_ERR, "rlm_nibs (nibs_filter): Couldn't get sum of week time - database query error `%s'", (inst->module->als_error)(sqlsocket, inst->config));
				return -1;
			}
			if(rlm_nibs_fetch_row(sqlsocket, inst))
			{
				radlog(L_ERR, "rlm_nibs (nibs_filter): Couldn't get sum of week time - SQL query did not succeed for user `%s'", request->username->strvalue);
				sql_mod_ok( inst, sqlsocket, NULL);
				return -1;
			}
			row = sqlsocket->row;
			nibs_user->week_time = Atoi(row[0]);
			sql_mod_ok( inst, sqlsocket, NULL);
			if (nibs_user->week_time_limit <= nibs_user->week_time)
				return 13;
			}
		else
			radlog(L_ERR,"rlm_nibs (nibs_filter): No week_time query configured");
	}
//Day time limit
	if (nibs_user->day_time_limit != 0)
	{
		DEBUG2("rlm_nibs (nibs_filter): Day time limit filter for user '%s'",  request->username->strvalue);
		radius_xlat(querystr, sizeof(querystr), inst->config->day_time_query, request, als_escape_func);
		if(*querystr)
		{
			sqlsocket = als_get_socket(inst);
			if (sqlsocket == NULL)
				return -1;
			query_log(request, inst, querystr);
			if (rlm_nibs_select_query(sqlsocket, inst, querystr))
			{
				radlog(L_ERR, "rlm_nibs (nibs_filter): Couldn't get sum of day time - database query error `%s'", (inst->module->als_error)(sqlsocket, inst->config));
				return -1;
			}
			if(rlm_nibs_fetch_row(sqlsocket, inst))
			{
				radlog(L_ERR, "rlm_nibs (nibs_filter): Couldn't get sum of day time - SQL query did not succeed for user `%s'", request->username->strvalue);
				sql_mod_ok( inst, sqlsocket, NULL);
				return -1;
			}
			row = sqlsocket->row;
			nibs_user->day_time = Atoi(row[0]);
			sql_mod_ok( inst, sqlsocket, NULL);
			if (nibs_user->day_time_limit <= nibs_user->day_time)
				return 14;
		}
		else
			radlog(L_ERR,"rlm_nibs (nibs_filter): No day_time query configured");
	}
//Total traffic limit
	if (nibs_user->total_traffic_limit != 0)
	{
		DEBUG2("rlm_nibs (nibs_filter): Total traffic limit filter for user '%s'",  request->username->strvalue);
		if (nibs_user->total_traffic_limit <= nibs_user->total_traffic)
			return 21;
	}
//Month traffic limit
	if (nibs_user->month_traffic_limit != 0)
	{
		DEBUG2("rlm_nibs (nibs_filter): Month traffic limit filter for user '%s'",  request->username->strvalue);
		radius_xlat(querystr, sizeof(querystr), inst->config->month_traffic_query, request, als_escape_func);
		if(*querystr)
		{
			sqlsocket = als_get_socket(inst);
			if (sqlsocket == NULL)
				return -1;
			query_log(request, inst, querystr);
			if (rlm_nibs_select_query(sqlsocket, inst, querystr))
			{
				radlog(L_ERR, "rlm_nibs (nibs_filter): Couldn't get sum of month traffic - database query error `%s'", (inst->module->als_error)(sqlsocket, inst->config));
				return -1;
			}
			if(rlm_nibs_fetch_row(sqlsocket, inst))
			{
				radlog(L_ERR, "rlm_nibs (nibs_filter): Couldn't get sum of month traffic - SQL query did not succeed for user `%s'", request->username->strvalue);
				sql_mod_ok( inst, sqlsocket, NULL);
				return -1;
			}
			row = sqlsocket->row;
			switch (nibs_user->direction)
			{
				case 1:
					tmpll = Atoll(row[0]);
				break;
				case 2:
					tmpll = Atoll(row[1]);
				break;
				case 3:
					tmpll = (Atoll(row[0]) + Atoll(row[1]));
				break;
				case 4:
					tmpll = (Atoll(row[0]) >= Atoll(row[1])) ? Atoll(row[0]) : Atoll(row[1]);
				break;
				case 5:
					tmpll = (Atoll(row[0]) <= Atoll(row[1])) ? Atoll(row[0]) : Atoll(row[1]);
				break;
				case 0:
				break;
				default:
					tmpll = (Atoll(row[0]) >= Atoll(row[1])) ? Atoll(row[0]) : Atoll(row[1]);
				break;
			}
			sql_mod_ok( inst, sqlsocket, NULL);
			nibs_user->month_traffic = tmpll;
			if (nibs_user->month_traffic_limit <= nibs_user->month_traffic)
				return 22;
		}
		else
			radlog(L_ERR,"rlm_nibs (nibs_filter): No month_traffic query configured");
	}
//Week traffic limit
	if (nibs_user->week_traffic_limit != 0)
	{
		DEBUG2("rlm_nibs (nibs_filter): Week traffic limit filter for user '%s'",  request->username->strvalue);
		radius_xlat(querystr, sizeof(querystr), inst->config->week_traffic_query, request, als_escape_func);
		if(*querystr)
		{
			sqlsocket = als_get_socket(inst);
			if (sqlsocket == NULL)
				return -1;
			query_log(request, inst, querystr);
			if (rlm_nibs_select_query(sqlsocket, inst, querystr))
			{
				radlog(L_ERR, "rlm_nibs (nibs_filter): Couldn't get sum of week traffic - database query error `%s'", (inst->module->als_error)(sqlsocket, inst->config));
				return -1;
			}
			if(rlm_nibs_fetch_row(sqlsocket, inst))
			{
				radlog(L_ERR, "rlm_nibs (nibs_filter): Couldn't get sum of week traffic - SQL query did not succeed for user `%s'", request->username->strvalue);
				sql_mod_ok( inst, sqlsocket, NULL);
				return -1;
			}
			row = sqlsocket->row;
			switch (nibs_user->direction)
			{
				case 1:
					tmpll = Atoll(row[0]);
				break;
				case 2:
					tmpll = Atoll(row[1]);
				break;
				case 3:
					tmpll = (Atoll(row[0]) + Atoll(row[1]));
				break;
				case 4:
					tmpll = (Atoll(row[0]) >= Atoll(row[1])) ? Atoll(row[0]) : Atoll(row[1]);
				break;
				case 5:
					tmpll = (Atoll(row[0]) <= Atoll(row[1])) ? Atoll(row[0]) : Atoll(row[1]);
				break;
				case 0:
				break;
				default:
					tmpll = (Atoll(row[0]) >= Atoll(row[1])) ? Atoll(row[0]) : Atoll(row[1]);
				break;
			}
			sql_mod_ok( inst, sqlsocket, NULL);
			nibs_user->week_traffic = tmpll;
			if (nibs_user->week_traffic_limit <= nibs_user->week_traffic)
				return 23;
		} 
		else
			radlog(L_ERR,"rlm_nibs (nibs_filter): No week_traffic query configured");
	}
//Day traffic limit
	if (nibs_user->day_traffic_limit != 0)
	{
		DEBUG2("rlm_nibs (nibs_filter): Day traffic limit filter for user '%s'",  request->username->strvalue);
		radius_xlat(querystr, sizeof(querystr), inst->config->day_traffic_query, request, als_escape_func);
		if(*querystr)
		{
			sqlsocket = als_get_socket(inst);
			if (sqlsocket == NULL)
				return -1;
			query_log(request, inst, querystr);
			if (rlm_nibs_select_query(sqlsocket, inst, querystr))
			{
				radlog(L_ERR, "rlm_nibs (nibs_filter): Couldn't get sum of day traffic - database query error `%s'", (inst->module->als_error)(sqlsocket, inst->config));
				return -1;
			}
			if(rlm_nibs_fetch_row(sqlsocket, inst))
			{
				radlog(L_ERR, "rlm_nibs (nibs_filter): Couldn't get sum of day traffic - SQL query did not succeed for user `%s'", request->username->strvalue);
				sql_mod_ok( inst, sqlsocket, NULL);
				return -1;
			}
			row = sqlsocket->row;
			switch (nibs_user->direction)
			{
				case 1:
					tmpll = Atoll(row[0]);
				break;
				case 2:
					tmpll = Atoll(row[1]);
				break;
				case 3:
					tmpll = (Atoll(row[0]) + Atoll(row[1]));
				break;
				case 4:
					tmpll = (Atoll(row[0]) >= Atoll(row[1])) ? Atoll(row[0]) : Atoll(row[1]);
				break;
				case 5:
					tmpll = (Atoll(row[0]) <= Atoll(row[1])) ? Atoll(row[0]) : Atoll(row[1]);
				break;
				case 0:
				break;
				default:
					tmpll = (Atoll(row[0]) >= Atoll(row[1])) ? Atoll(row[0]) : Atoll(row[1]);
				break;
			}
			sql_mod_ok( inst, sqlsocket, NULL);
			nibs_user->day_traffic = tmpll;
			if (nibs_user->day_traffic_limit <= nibs_user->day_traffic)
				return 24;
		}
		else
			radlog(L_ERR,"rlm_nibs (nibs_filter): No day_traffic query configured");
	}
//Login time
	if (nibs_user->login_time[0] != 0)
	{
		DEBUG2("rlm_nibs (nibs_filter): Login time filter for user '%s'",  request->username->strvalue);
		if (Timestr_match(nibs_user->login_time, request->timestamp, inst) < 0)
			return 40;
	}
//Black list
	if((inst->config->use_black_list) && (nibs_find_in_blacklist((uid_t)nibs_user->uid, (gid_t)nibs_user->gid, nibs_user->call_from, inst, request)))
		return 50;
	DEBUG2("rlm_nibs (nibs_filter): end for user '%s' ------------",  request->username->strvalue);
	return 0;
}

/*************************************************************************
 *
 *	Function: nibs_decode_filter_error
 *
 *	Purpose: Decode an errorr that returns by funnction nibs_filter
 *
 *************************************************************************/
char *nibs_decode_filter_error(int nibs_errno, REQUEST * request)
{
	char	*nibs_reply,
		*nibs_reason;
	VALUE_PAIR	*vp;
    
	switch (nibs_errno)
	{
	case -1:
		nibs_reason = strdup("nibs_user structure do not filled");
		nibs_reply = NULL;
	break;
	case 0:
		nibs_reason = strdup("successfully authenticated via sql");
		nibs_reply = NULL;
	break;
	case 1:
		nibs_reason = strdup("no money on deposit");
		nibs_reply = strdup("you have no money on your deposit");
	break;
	case 2:
		nibs_reason = strdup("account is expired");
		nibs_reply = strdup("your account is expired");
	break;
	case 3:
		nibs_reason = strdup("account is blocked");
		nibs_reply = strdup("your account is deactivated");
	break;
	case 11:
		nibs_reason = strdup("total limit of time is reached");
		nibs_reply = strdup(nibs_reason);
	break;
	case 12:
		nibs_reason = strdup("month limit of time is reached");
		nibs_reply = strdup(nibs_reason);
	break;
	case 13:
		nibs_reason = strdup("week limit of time is reached");
		nibs_reply = strdup(nibs_reason);
	break;
	case 14:
		nibs_reason = strdup("day limit of time is reached");
		nibs_reply = strdup(nibs_reason);
	break;
	case 21:
		nibs_reason = strdup("total limit of traffic is reached");
		nibs_reply = strdup(nibs_reason);
	break;
	case 22:
		nibs_reason = strdup("month limit of traffic is reached");
		nibs_reply = strdup(nibs_reason);
	break;
	case 23:
		nibs_reason = strdup("week limit of traffic is reached");
		nibs_reply = strdup(nibs_reason);
	break;
	case 24:
		nibs_reason = strdup("day limit of traffic is reached");
		nibs_reply = strdup(nibs_reason);
	break;
	case 31:
		nibs_reason = strdup("activation time in future");
		nibs_reply = strdup("your account is not activated today");
	    break;
	case 40:
		nibs_reason = strdup("called outside allowed timespan");
		nibs_reply = strdup("you are calling outside your allowed timespan");
	break;
	case 50:
		nibs_reason = strdup("access from this phone number is disabled");
		nibs_reply = strdup(nibs_reason);
	break;
	case 61:
		nibs_reason = strdup("total limit of money is reached");
		nibs_reply = strdup(nibs_reason);
	break;
	case 62:
		nibs_reason = strdup("month limit of money is reached");
		nibs_reply = strdup(nibs_reason);
	break;
	case 63:
		nibs_reason = strdup("week limit of money is reached");
		nibs_reply = strdup(nibs_reason);
	break;
	case 64:
		nibs_reason = strdup("day limit of money is reached");
		nibs_reply = strdup(nibs_reason);
	break;
	default:
		nibs_reason = strdup("Unknown error");
		nibs_reply = NULL;
	break;
	}
	if(nibs_reply != NULL)
	{
		pairdelete(&request->reply->vps, PW_REPLY_MESSAGE);
		vp = pairmake("Reply-Message", nibs_reply, T_OP_EQ);
		if (vp == NULL)
			radlog(L_ERR, "%s", librad_errstr);
		else
			pairadd(&request->reply->vps, vp);
		free(nibs_reply);
	}
	return nibs_reason;
}

/*************************************************************************
 *
 *	Function: nibs_user_add_acct
 *
 *	Purpose: create and fill NIBS_USER structure by the SQL values
 *	Return -1 if error
 *		0 if statistic not found
 *		1 if all ok
 *
 *************************************************************************/
int nibs_user_add_acct(NIBS_USER *nibs_user, void *instance, REQUEST * request)
{
	SQLSOCK	*sqlsocket = NULL;
	SQL_ROW	row;
	SQL_INST	*inst = instance;
	char	querystr[MAX_QUERY_LEN];

	if (nibs_user == NULL)
		return -1;
	memset(querystr, 0, MAX_QUERY_LEN);
	radius_xlat(querystr, MAX_QUERY_LEN, inst->config->statistic_select_query, request, als_escape_func);
	if (*querystr)
	{
		sqlsocket = als_get_socket(inst);
		if (sqlsocket == NULL)
			return -1;
		query_log(request, inst, querystr);
		if (rlm_nibs_select_query(sqlsocket, inst, querystr))
		{
			snprintf(querystr, sizeof(querystr), "rlm_nibs (nibs_user_add_acct): database query error - %s" ,(inst->module->als_error)(sqlsocket, inst->config));
			return sql_mod_fail(inst, sqlsocket, querystr);
		}
		if ((inst->module->als_num_rows)(sqlsocket, inst->config) == 0)
		{
			sql_mod_ok( inst, sqlsocket, NULL);
			return 0;
		}
		if(rlm_nibs_fetch_row(sqlsocket, inst))
		{
			DEBUG("rlm_nibs (nibs_user_add_acct): SQL query did not succeed for user `%s'", request->username->strvalue);
			sql_mod_ok( inst, sqlsocket, NULL);
			return 0;
		}
		row = sqlsocket->row;
		nibs_user->time_on = Atoll(row[0]);
		nibs_user->in_bytes = Atoll(row[1]);
		nibs_user->out_bytes = Atoll(row[2]);
		nibs_user->start_time = Atoi(row[3]);
		nibs_user->stop_time = (nibs_user->current_time_on > 0) ? nibs_user->start_time + nibs_user->current_time_on : 0;
		nibs_user->money = Atof(row[4]);
                nibs_user->hour_in_bytes = Atoll(row[5]);
		nibs_user->hour_out_bytes = Atoll(row[6]);
		nibs_user->hour_traffic_money = Atof(row[7]);
		nibs_user->last_change = Atoi(row[8]);
		sql_mod_ok( inst, sqlsocket, NULL);
		return 1;
	}
	else
		radlog(L_ERR, "rlm_nibs (nibs_user_add_acct): no `statistic_select_query' configured");
	return -1;
}

/*************************************************************************
 *
 *	Function: nibs_fill_user
 *
 *	Purpose: create and fill NIBS_USER structure by the SQL values
 *
 *************************************************************************/
NIBS_USER *nibs_fill_user(void *instance, REQUEST * request)
{
	SQLSOCK		*sqlsocket = NULL;
	SQL_ROW		row;
	VALUE_PAIR	*vp;
	SQL_INST	*inst = instance;
	NIBS_USER	*nibs_user;
	PACKET		*user_packet;
	char		querystr[MAX_QUERY_LEN];
	char		querystr2[MAX_QUERY_LEN];
	char		*call_from = NULL;
    
	DEBUG2("rlm_nibs (nibs_fill_user): begin for user `%s' ------------", request->username->strvalue);
	if ((vp = pairfind(request->packet->vps, PW_CALLING_STATION_ID)) != NULL)
		call_from = vp->strvalue;
	memset(querystr, 0, MAX_QUERY_LEN);
	radius_xlat(querystr, MAX_QUERY_LEN, inst->config->authorize_query, request, als_escape_func);
	if (*querystr)
	{
		sqlsocket = als_get_socket(inst);
		if (sqlsocket == NULL)
			return NULL;
		query_log(request, inst, querystr);
		if (rlm_nibs_select_query(sqlsocket, inst, querystr))
		{
			radlog(L_ERR, "rlm_nibs (nibs_fill_user): database query error - `%s'", (inst->module->als_error)(sqlsocket, inst->config));
			als_release_socket(inst, sqlsocket);
			return NULL;
		}
		if ((inst->module->als_num_rows)(sqlsocket, inst->config) == 0)
		{
			radius_xlat(querystr2, sizeof(querystr2), "rlm_nibs (nibs_fill_user): User `%{SQL-User-Name}' not found in database [%{NAS-IP-Address}:%{NAS-Port}]%%s%%s%%s", request, als_escape_func);
			snprintf(querystr, sizeof(querystr), querystr2, (call_from) ? " |" : "", (call_from) ? call_from : "", (call_from) ? "|" : "");
			radlog(L_AUTH, querystr);
			sql_mod_ok( inst, sqlsocket, NULL);
			return NULL;
		}
		if(rlm_nibs_fetch_row(sqlsocket, inst))
		{
			DEBUG("rlm_nibs (nibs_fill_user): SQL query did not succeed for user `%s'", request->username->strvalue);
			sql_mod_ok( inst, sqlsocket, NULL);
			return NULL;
		}
		row = sqlsocket->row;
		if ((user_packet = nibs_packet_find(inst, Atoll(row[4]))) == NULL)
		{
			radlog(L_ERR, "rlm_nibs (nibs_fill_user): No packet (group) for user '%s' with gid = %d", request->username->strvalue, Atoi(row[4]));
			sql_mod_ok( inst, sqlsocket, NULL);
			return NULL;
		}
		if ((nibs_user = rad_malloc(sizeof(NIBS_USER))) == NULL)
		{
			radlog(L_ERR, "rlm_nibs (nibs_fill_user): out of memory");
			sql_mod_ok( inst, sqlsocket, NULL);
			return NULL;
		}
		memset(nibs_user, 0, sizeof(NIBS_USER));
		if ((vp = pairfind(request->packet->vps, PW_CALLING_STATION_ID)) != NULL)
			strNcpy(nibs_user->call_from, vp->strvalue, sizeof(nibs_user->call_from));
		else
			nibs_user->call_from[0] = 0;
		if ((vp = pairfind(request->packet->vps, PW_NAS_IP_ADDRESS)) != NULL)
		{
			strNcpy(nibs_user->nas_address, vp->strvalue, sizeof(nibs_user->nas_address));
			nibs_user->nas_ip = vp->lvalue;
		}
		/* User uniq values */
		if (row[0] && (row[0][0] != 0)) strNcpy(nibs_user->name, row[0], sizeof(nibs_user->name)); else nibs_user->name[0] = 0;
		if (row[1] && (row[1][0] != 0)) strNcpy(nibs_user->passwd, row[1], sizeof(nibs_user->passwd)); else nibs_user->passwd[0] = 0;
		nibs_user->crypt_method = Atoi(row[2]);
		nibs_user->uid = Atoll(row[3]);
		nibs_user->gid = Atoll(row[4]);
		nibs_user->deposit = Atof(row[5]);
		nibs_user->credit = Atof(row[6]);
		nibs_user->add_date = Atoi(row[7]);
		nibs_user->activated = Atoi(row[9]);
		nibs_user->expired = Atoi(row[10]);
		nibs_user->total_time = Atoll(row[11]);
		nibs_user->total_traffic = Atoll(row[12]);
		nibs_user->total_money = Atof(row[13]);
		nibs_user->last_connection = Atoi(row[14]);
		if (row[17] && (row[17][0] != 0))
			strNcpy(nibs_user->callback_number, row[17], sizeof(nibs_user->callback_number));
		else
			nibs_user->callback_number[0] = 0;
		/* Packet default values */
		if (user_packet->prefix[0] != 0)
			strNcpy(nibs_user->prefix, user_packet->prefix, sizeof(nibs_user->prefix));
		else
			nibs_user->prefix[0] = 0;
		nibs_user->tos = user_packet->tos;
		nibs_user->do_with_tos = user_packet->do_with_tos;
		nibs_user->direction = user_packet->direction;
		nibs_user->fixed = user_packet->fixed;
		nibs_user->fixed_cost = user_packet->fixed_cost;
		nibs_user->activation_time = user_packet->activation_time;
		nibs_user->total_time_limit = user_packet->total_time_limit;
		nibs_user->month_time_limit = user_packet->month_time_limit;
		nibs_user->week_time_limit = user_packet->week_time_limit;
		nibs_user->day_time_limit = user_packet->day_time_limit;
		nibs_user->total_traffic_limit = user_packet->total_traffic_limit;
		nibs_user->month_traffic_limit = user_packet->month_traffic_limit;
		nibs_user->week_traffic_limit = user_packet->week_traffic_limit;
		nibs_user->day_traffic_limit = user_packet->day_traffic_limit;
		nibs_user->total_money_limit = user_packet->total_money_limit;
		nibs_user->month_money_limit = user_packet->month_money_limit;
		nibs_user->week_money_limit = user_packet->week_money_limit;
		nibs_user->day_money_limit = user_packet->day_money_limit;
		nibs_user->allow_callback = user_packet->allow_callback;
		nibs_user->session_timeout = user_packet->session_timeout;
		if (user_packet->other_params[0] != 0)
			strNcpy(nibs_user->other_params, user_packet->other_params, sizeof(nibs_user->other_params));
		else
			nibs_user->other_params[0] = 0;
		if (user_packet->huntgroup_name[0] != 0)
			strNcpy(nibs_user->huntgroup_name, user_packet->huntgroup_name, sizeof(nibs_user->huntgroup_name));
		else	
			nibs_user->huntgroup_name[0] = 0;
		nibs_user->idle_timeout = user_packet->idle_timeout;
		if (user_packet->allowed_prefixes[0] != 0)
			strNcpy(nibs_user->allowed_prefixes, user_packet->allowed_prefixes, sizeof(nibs_user->allowed_prefixes));
		else
			nibs_user->allowed_prefixes[0] = 0;
		nibs_user->simultaneous_use = user_packet->simultaneous_use;
		nibs_user->port_limit = user_packet->port_limit;
		if (user_packet->login_time[0] != 0)
			strNcpy(nibs_user->login_time, user_packet->login_time, sizeof(nibs_user->login_time));
		else
			nibs_user->login_time[0] = 0;
		if (user_packet->framed_ip[0] != 0)
			strNcpy(nibs_user->framed_ip, user_packet->framed_ip, sizeof(nibs_user->framed_ip));
		else
			nibs_user->framed_ip[0] = 0;
		if (user_packet->framed_mask[0] != 0)
			strNcpy(nibs_user->framed_mask, user_packet->framed_mask, sizeof(nibs_user->framed_mask));
		else
			nibs_user->framed_mask[0] = 0;
		nibs_user->no_pass = user_packet->no_pass;
		nibs_user->no_acct = user_packet->no_acct;
		/* User additional values, that make raplace packet default values */
		if (nibs_user->blocked == 0)
			nibs_user->blocked = Atoi(row[8]);
		if (row[15] && (row[15][0] != 0))
			strNcpy(nibs_user->framed_ip, row[15], sizeof(nibs_user->framed_ip));
		if (row[16] && (row[16][0] != 0))
			strNcpy(nibs_user->framed_mask, row[16], sizeof(nibs_user->framed_mask));
		sql_mod_ok( inst, sqlsocket, NULL);
		/* Prof mode */
		memset(querystr, 0, MAX_QUERY_LEN);
		radius_xlat(querystr, MAX_QUERY_LEN, inst->config->authorize_query_prof, request, als_escape_func);
		if (*querystr)
		{
			DEBUG2("rlm_nibs (nibs_fill_user): ----- prof mode begin for user `%s' -----", request->username->strvalue);
			sqlsocket = als_get_socket(inst);
			if (sqlsocket == NULL)
				return nibs_user;
			query_log(request, inst, querystr);
			if (rlm_nibs_select_query(sqlsocket, inst, querystr))
			{
				radlog(L_ERR, "rlm_nibs (nibs_fill_user): database query error - `%s'", (inst->module->als_error)(sqlsocket, inst->config));
				als_release_socket(inst, sqlsocket);
				return nibs_user;
			}
			if ((inst->module->als_num_rows)(sqlsocket, inst->config) == 0)
			{
				radlog(L_AUTH, "rlm_nibs (nibs_fill_user): prof_User `%s' not found in database", request->username->strvalue);
				sql_mod_ok( inst, sqlsocket, NULL);
				return nibs_user;
			}
			if(rlm_nibs_fetch_row(sqlsocket, inst))
			{
				DEBUG("rlm_nibs (nibs_fill_user): SQL query did not succeed");
				sql_mod_ok( inst, sqlsocket, NULL);
				return nibs_user;
			}
			row = sqlsocket->row;
			if (row[0])
				nibs_user->tos = Atoi(row[0]);
			if (row[1])
				nibs_user->do_with_tos = Atoi(row[1]);
			if (row[2])
				nibs_user->direction = Atoi(row[2]);
			if (row[3]) 
				nibs_user->fixed = Atoi(row[3]);
			if (row[4])
				nibs_user->fixed_cost = Atof(row[4]);
			if (row[5])
				nibs_user->activation_time = Atoi(row[5]);
			if (row[6])
				nibs_user->total_time_limit = Atoll(row[6]);
			if (row[7])
				nibs_user->month_time_limit = Atoi(row[7]);
			if (row[8])
				nibs_user->week_time_limit = Atoi(row[8]);
			if (row[9])
				nibs_user->day_time_limit = Atoi(row[9]);
			if (row[10])
				nibs_user->total_traffic_limit = Atoll(row[10]);
			if (row[11])
				nibs_user->month_traffic_limit = Atoll(row[11]);
			if (row[12])
				nibs_user->week_traffic_limit = Atoll(row[12]);
			if (row[13])
				nibs_user->day_traffic_limit = Atoll(row[13]);
			if (row[14])
				nibs_user->total_money_limit = Atof(row[14]);
			if (row[15])
				nibs_user->month_money_limit = Atof(row[15]);
			if (row[16])
				nibs_user->week_money_limit = Atof(row[16]);
			if (row[17])
				nibs_user->day_money_limit = Atof(row[17]);
			if (row[18] && (row[18][0] != 0))
				strNcpy(nibs_user->login_time,row[18],sizeof(nibs_user->login_time));
			if (row[19] && (row[19][0] != 0))
				strNcpy(nibs_user->huntgroup_name, row[19], sizeof(nibs_user->huntgroup_name));
			if (row[20])
				nibs_user->simultaneous_use = Atoi(row[20]);
			if (row[21])
				nibs_user->port_limit = Atoi(row[21]);
			if (row[22])
				nibs_user->session_timeout = Atoi(row[22]);
			if (row[23])
				nibs_user->idle_timeout = Atoi(row[23]);
			if (row[24] && (row[24][0] != 0))
				strNcpy(nibs_user->allowed_prefixes, row[24], sizeof(nibs_user->allowed_prefixes));
			if (row[25])
				nibs_user->no_pass = Atoi(row[25]);
			if (row[26])
				nibs_user->no_acct = Atoi(row[26]);
			if (row[27])
				nibs_user->allow_callback = Atoi(row[27]);
			if (row[28] && (row[28][0] != 0))
				strNcpy(nibs_user->other_params, row[28], sizeof(nibs_user->other_params));
			sql_mod_ok( inst, sqlsocket, NULL);
			DEBUG2("rlm_nibs (nibs_fill_user): ----- prof mode end for user `%s' -----", request->username->strvalue);
			}
		DEBUG2("rlm_nibs (nibs_fill_user): end for user `%s' ------------", request->username->strvalue);
		return nibs_user;
		}
	radlog(L_ERR, "rlm_nibs (nibs_fill_user): no `authorize_query' configured");
	return NULL;
}

/*
 *	Get all groups, holidays and prices into the memory.
 */
int nibs_init(void *instance)
{
	SQL_INST	*inst = instance;

	radlog(L_INFO, "rlm_nibs (nibs_init): Initializing main structures `%s'", inst->config->nibs_db);
	if (sql_init_groups(inst) < 0)
	{
		return -1;
	}
	if (inst->config->packets > 0)
		radlog(L_INFO, "rlm_nibs (nibs_init): Initializing group table `%s' done. Found %d groups", inst->config->nibs_packets_table, inst->config->packets);

	if (sql_init_holidays(inst) < 0)
	{
		return -1;
	}
	if (inst->config->holidays > 0)
		radlog(L_INFO, "rlm_nibs (nibs_init): Initializing holiday table `%s' done. Found %d holidays", inst->config->nibs_holidays_table, inst->config->holidays);
	if (sql_init_prices(inst) < 0)
	{
		return -1;
	}
	if (inst->config->num_prices > 0)
		radlog(L_INFO, "rlm_nibs (nibs_init): Initializing prices table `%s' done. Found %d prices", inst->config->nibs_prices_table, inst->config->num_prices);
	return 0;
}

int nibs_free(void *instance)
{
	SQL_INST	*inst = instance;

	cleanup_holidays( inst);
	cleanup_prices( inst);
	cleanup_groups( inst);
	return 0;
}

/*************************************************************************
 *
 *	Function: nibs_find_in_blacklist
 *
 *	Purpose: Return 1 if number in black list or 0 if no
 *
 *************************************************************************/
int nibs_find_in_blacklist(ULONGINT uid, ULONGINT gid, char *tel_num, void *instance, REQUEST * request)
{
    	SQLSOCK 	*sqlsocket;
	SQL_INST	*inst = instance;
	SQL_ROW		row;
	char		querystr[MAX_QUERY_LEN];
	int		default_action = 0;
	int		action = 0,
			found = 0;
	ULONGINT	ruid;
	ULONGINT	rgid;

	DEBUG2("rlm_nibs (nibs_find_in_blacklist): Blacklist filter for user '%s'",  request->username->strvalue);

	if(tel_num[0] == 0)
	{
		DEBUG2("rlm_nibs (nibs_find_in_blacklist): No telephone number for user '%s'",  request->username->strvalue);
		return 0;
	}
	
	radius_xlat(querystr, MAX_QUERY_LEN, inst->config->blacklist_select_query, request, als_escape_func);

	if(*querystr)
	{
		sqlsocket = als_get_socket(inst);
		if (sqlsocket == NULL)
			return -1;
/* global */
		DEBUG2("rlm_nibs (nibs_find_in_blacklist): Checking global phone access right");
		query_log(request, inst, querystr);

		if (rlm_nibs_select_query(sqlsocket, inst, querystr))
			radlog(L_ERR, "rlm_nibs (nibs_find_in_blacklist): 1) Couldn't get blacklist entries - %s", (inst->module->als_error)(sqlsocket, inst->config));
		default_action = action = found = 0;
		while(rlm_nibs_fetch_row(sqlsocket, inst) == 0)
		{
			row = sqlsocket->row;
			if (row == NULL)
				break;
			ruid = (row[0] == NULL) ? (ULONGINT)-1L : (ULONGINT)Atoll(row[0]);
			rgid = (row[1] == NULL) ? (ULONGINT)-1L : (ULONGINT)Atoll(row[1]);
			if ((row[0] == NULL) && (row[1] == NULL))
			{
				/* found global */
				if (as_bool((int)row[3][0]))
				{
					DEBUG2("rlm_nibs (nibs_find_in_blacklist): Using allowed phones - setting default actions to deny");
					default_action = 1;	// Not allowed by default
				}
				DEBUG2("rlm_nibs (nibs_find_in_blacklist): Check phone access for phone '%s' and pattern '%s'", tel_num, row[2]);
				if ((strcmp_st( row[2], tel_num)) && (found == 0))
				{
					found = 1;
					if (as_bool((int)row[3][0]))
					{
						DEBUG2("rlm_nibs (nibs_find_in_blacklist): Access permitted from phone '%s'", tel_num);
						action = 0;	// Permit access from this N
					}
					else
					{
						DEBUG2("rlm_nibs (nibs_find_in_blacklist): Access deny from phone '%s'",  tel_num);
						action = 1;
					}
				}
			}
		}
		(inst->module->als_finish_select_query)(sqlsocket, inst->config);
/* group */
		DEBUG2("rlm_nibs (nibs_find_in_blacklist): Checking group phone access right");
		query_log(request, inst, querystr);
		if (rlm_nibs_select_query(sqlsocket, inst, querystr))
			radlog(L_ERR, "rlm_nibs (nibs_find_in_blacklist): 2) Couldn't get blacklist entries - %s", (inst->module->als_error)(sqlsocket, inst->config));
		while(rlm_nibs_fetch_row(sqlsocket, inst) == 0)
		{
			row = sqlsocket->row;
			if (row == NULL)
				break;
			ruid = (row[0] == NULL) ? (ULONGINT)-1L : (ULONGINT)Atoll(row[0]);
			rgid = (row[1] == NULL) ? (ULONGINT)-1L : (ULONGINT)Atoll(row[1]);
			if ((ruid == (ULONGINT)-1L) && (rgid == gid))
			{
				/* found group */
				if (as_bool((int)row[3][0]))
				{
					DEBUG2("rlm_nibs (nibs_find_in_blacklist): Using allowed phones - setting default actions to deny");
					default_action = 1;	// Not allowed by default
				}
				DEBUG2("rlm_nibs (nibs_find_in_blacklist): Check phone access for phone '%s' and pattern '%s'", 	tel_num, row[2]);
				if ((strcmp_st( row[2], tel_num)) && (found < 2))
				{
					found = 2;
					if (as_bool((int)row[3][0]))
					{
						DEBUG2("rlm_nibs (nibs_find_in_blacklist): Access permitted from phone '%s'",  tel_num);
						action = 0;	// Permit access from this N
					}
					else
					{
						DEBUG2("rlm_nibs (nibs_find_in_blacklist): Access deny from phone '%s'",  tel_num);
						action = 1;
					}
				}
			}
		}
		(inst->module->als_finish_select_query)(sqlsocket, inst->config);
/* user */
		DEBUG2("rlm_nibs (nibs_find_in_blacklist): Checking user phone access right");
		query_log(request, inst, querystr);
		if (rlm_nibs_select_query(sqlsocket, inst, querystr))
			radlog(L_ERR, "rlm_nibs (nibs_find_in_blacklist): 3) Couldn't get blacklist entries - %s", (inst->module->als_error)(sqlsocket, inst->config));
		while(rlm_nibs_fetch_row(sqlsocket, inst) == 0)
		{
			row = sqlsocket->row;
			if (row == NULL)
				break;
			ruid = (row[0] == NULL) ? (ULONGINT)-1L : (ULONGINT)Atoll(row[0]);
			rgid = (row[1] == NULL) ? (ULONGINT)-1L : (ULONGINT)Atoll(row[1]);
			if ((ruid == uid) && (rgid == (ULONGINT)-1L))
			{
				/* found user */
				if (as_bool((int)row[3][0]))
				{
					DEBUG2("rlm_nibs (nibs_find_in_blacklist): Using allowed phones - setting default actions to deny");
					default_action = 1;	// Not allowed by default
				}
				DEBUG2("rlm_nibs (nibs_find_in_blacklist): Check phone access for phone '%s' and pattern '%s'", 	tel_num, row[2]);
				if ((strcmp_st( row[2], tel_num)) && (found < 4))
				{
					found = 4;
					if (as_bool((int)row[3][0]))
					{
						DEBUG2("rlm_nibs (nibs_find_in_blacklist): Access permitted from phone '%s'",  tel_num);
						action = 0;	// Permit access from this N
					}
					else
					{
						DEBUG2("rlm_nibs (nibs_find_in_blacklist): Access deny from phone '%s'",  tel_num);
						action = 1;
					}
				}
			}
		}
		sql_mod_ok(inst,sqlsocket,NULL);
	}
	else
		radlog(L_ERR, "rlm_nibs (nibs_fill_user): no `blacklist_select_query' configured");
	if (found > 0)
	{
		DEBUG("rlm_nibs (nibs_find_in_blacklist): Found rule, access '%s'",  (action == 0) ? "permitted" : "deny");
		return action;
	}
	else
	{
		DEBUG("rlm_nibs (nibs_find_in_blacklist): Rule not found, use default - access '%s'",  (default_action == 0) ? "permitted" : "deny");
		return default_action;
	}
}
