/***************************************************************************
*  nibs.h                              rlm_nibs - FreeRADIUS SQL Module    *
*                                                                          *
*      Header for main library file                                        *
*                                                                          *
*                               Neon aka Sergey Derkach <neon@is.kh.ua>    *
***************************************************************************/
#ifndef _nibs_H
#define _nibs_H

#include "radiusd.h"

#define MAX_EXPIRE 604800

#define	ULONGINT	unsigned long int

typedef struct nibs_user {
    char	    name[65];
    char	    prefix[5];
    char	    passwd[255];
    int		    crypt_method;
    ULONGINT	    gid;
    ULONGINT	    uid;
    double	    deposit;
    double	    money;
    double	    credit;
    int		    tos;
    int		    do_with_tos;
    int		    direction;
    int		    fixed;
    double	    fixed_cost;
    int	    	    add_date;
    int    	    blocked;
    int    	    activated;
    int		    activation_time;
    int	    	    expired;
    ULONGINT   	    total_time;
    int	    	    month_time;
    int	    	    week_time;
    int	    	    day_time;
    ULONGINT	    total_traffic;
    ULONGINT	    month_traffic;
    ULONGINT	    week_traffic;
    ULONGINT	    day_traffic;
    double	    total_money;
    double	    month_money;
    double	    week_money;
    double	    day_money;
    double	    last_connection;
    char	    framed_ip[17];
    char	    framed_mask[17];
    char	    callback_number[33];
    ULONGINT	    total_time_limit;
    int		    month_time_limit;
    int		    week_time_limit;
    int		    day_time_limit;
    ULONGINT	    total_traffic_limit;
    ULONGINT	    month_traffic_limit;
    ULONGINT	    week_traffic_limit;
    ULONGINT	    day_traffic_limit;
    double	    total_money_limit;
    double	    month_money_limit;
    double	    week_money_limit;
    double	    day_money_limit;
    int		    allow_callback;
    int		    session_timeout;
    char	    other_params[255];
    char	    huntgroup_name[65];
    int		    idle_timeout;
    char	    allowed_prefixes[33];
    int		    simultaneous_use;
    int		    port_limit;
    char	    login_time[128];
    int		    no_pass;
    int		    no_acct;
    char	    call_from[33];
    ULONGINT	    time_on;
    ULONGINT	    in_bytes;
    ULONGINT	    out_bytes;
    int             start_time;
    int             stop_time;

    ULONGINT	    hour_in_bytes;
    ULONGINT	    hour_out_bytes;
    double	    hour_traffic_money;
    int		    last_change;

    ULONGINT	    current_hour_in_bytes;
    ULONGINT	    current_hour_out_bytes;
    double	    current_hour_traffic_money;

    ULONGINT        current_time_on;
    ULONGINT	    current_in_bytes;
    ULONGINT	    current_out_bytes;
    ULONGINT   	    sub_total_time;
    ULONGINT	    sub_total_traffic;
    double	    sub_total_money;
    double	    sub_money;
    uint32_t	    nas_ip;
    char	    nas_address[16];
} NIBS_USER;


typedef struct packet {
    ULONGINT        gid;
    char            prefix[5];
    int             tos;
    int		    do_with_tos;
    int             direction;
    int             activation_time;
    ULONGINT        total_time_limit;
    int             month_time_limit;
    int             week_time_limit;
    int             day_time_limit;
    ULONGINT	    total_traffic_limit;
    ULONGINT	    month_traffic_limit;
    ULONGINT	    week_traffic_limit;
    ULONGINT	    day_traffic_limit;
    double	    total_money_limit;
    double	    month_money_limit;
    double	    week_money_limit;
    double	    day_money_limit;
    int		    fixed;
    double	    fixed_cost;
    int             allow_callback;
    int             session_timeout;
    char            other_params[255];
    char	    huntgroup_name[65];
    int             idle_timeout;
    char            allowed_prefixes[33];
    int             simultaneous_use;
    int		    port_limit;
    char            login_time[128];
    char	    framed_ip[17];
    char	    framed_mask[17];
    int             no_pass;
    int             no_acct;
    struct packet   *next;
} PACKET;

typedef struct prices {
    ULONGINT        gid;
    int             week_day;
    double          h0;
    double          input0;
    double          output0;
    double          h1;
    double          input1;
    double          output1;
    double          h2;
    double          input2;
    double          output2;
    double          h3;
    double          input3;
    double          output3;
    double          h4;
    double          input4;
    double          output4;
    double          h5;
    double          input5;
    double          output5;
    double          h6;
    double          input6;
    double          output6;
    double          h7;
    double          input7;
    double          output7;
    double          h8;
    double          input8;
    double          output8;
    double          h9;
    double          input9;
    double          output9;
    double          h10;
    double          input10;
    double          output10;
    double          h11;
    double          input11;
    double          output11;
    double          h12;
    double          input12;
    double          output12;
    double          h13;
    double          input13;
    double          output13;
    double          h14;
    double          input14;
    double          output14;
    double          h15;
    double          input15;
    double          output15;
    double          h16;
    double          input16;
    double          output16;
    double          h17;
    double          input17;
    double          output17;
    double          h18;
    double          input18;
    double          output18;
    double          h19;
    double          input19;
    double          output19;
    double          h20;
    double          input20;
    double          output20;
    double          h21;
    double          input21;
    double          output21;
    double          h22;
    double          input22;
    double          output22;
    double          h23;
    double          input23;
    double          output23;
    struct prices 	   *next;
} PRICES;

typedef struct holiday {
    char            holiday_date[6];
    struct holiday *next;
} HOLIDAY;

int		nibs_user_kill(void *instance, REQUEST * request);
int		nibs_billing(NIBS_USER *nibs_user, void *instance, REQUEST * request, int acctstatustype);
int		nibs_set_session_timeout(NIBS_USER *nibs_user, void *instance, time_t tso);
int		nibs_add_attrs(NIBS_USER *nibs_user, void *instance, REQUEST * request);
int		nibs_filter(NIBS_USER *nibs_user, void *instance, REQUEST * request);
char	       *nibs_decode_filter_error(int nibs_errno, REQUEST * request);
int		nibs_user_add_acct(NIBS_USER *nibs_user, void *instance, REQUEST * request);
void            nibs_add_current(NIBS_USER *nibs_user, REQUEST * request);
NIBS_USER      *nibs_fill_user(void *instance, REQUEST * request);
int             nibs_init(void *instance);
int             sql_init_holidays(void *instance);
int		nibs_is_holiday(char *dbuf, void *instance);
void		cleanup_holidays( void *);
int             sql_init_prices(void *instance);
double		nibs_find_time_cost(ULONGINT gid, int wday, int hour, void *instance);
double		nibs_find_traffic_cost(ULONGINT gid, int wday, int hour, int direction, void *instance);
void		cleanup_prices( void *);
int             sql_init_groups(void *instance);
PACKET	       *nibs_packet_find(void *instance, ULONGINT gid);
void		cleanup_groups( void *);
int		nibs_find_in_blacklist(ULONGINT uid, ULONGINT gid, char *tel_num, void *instance, REQUEST * request);
int		nibs_free(void *instance);
#endif
