#include	"autoconf.h"
#include	"libradius.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "nibs.h"
#include "rlm_nibs.h"
#include "nibs_utils.h"

static const char *Days[] = { "su", "mo", "tu", "we", "th", "fr", "sa", "wk", "hl", "any", "al" };
static const char *Pap_hextab = "0123456789abcdef";

#define DAYMIN		(24*60)
#define WEEKMIN		(24*60*7)
#define val(x)		(( (x) < 48 || (x) > 57) ? 0 : ((x) - 48))

#if 0 /* Set to 1 if you're a developer and want to debug this code */
#	define timestr_debug DEBUG2
#	define do_timestr_debug 1
#else
#	define timestr_debug if (0) printf
#endif


int Atoi(const char *value)
{
	return (value) ? (int)(atoi(value)) : 0;
}

long int Atol(const char *value)
{
	return (value) ? (long int)(atol(value)) : 0;
}

ULONGINT Atoll(const char *value)
{
#ifdef __linux__
	return (value) ? (unsigned long int)atoll(value) : 0;
#else
	return (value) ? (unsigned long int)atol(value) : 0;
#endif
}

double Atof(const char *value)
{
	return (value) ? (double)(atof(value)) : 0;
}

int Round(double value)
{
	return (int)value;
}

/*
 *	String code.
 */
static int Strcode (const char **str)
{
	int i;
	size_t l;

	timestr_debug("strcode %s called\n", *str);

	for (i = 0; i < 11; i++) 
	{
		l = strlen(Days[i]);
		if (l > strlen(*str))
			continue;
		if (strncmp(*str, Days[i], l) == 0) 
		{
			*str += l;
			break;
		}
	}
	timestr_debug("strcode result %d\n", i);

	return (i >= 11) ? -1 : i;

}

/*
 *	Fill bitmap with hours/mins.
 */
static int Hour_fill(char *bitmap, const char *tm)
{
	char *p;
	int start, end;
	int i, bit, byte;

	timestr_debug("hour_fill called for %s\n", tm);

	/*
	 *	Get timerange in start and end.
	 */
	end = -1;
	if ((p = strchr(tm, '-')) != NULL)
	{
		p++;
		if (p - tm != 5 || strlen(p) < 4 || !isdigit((int) *p))
			return 0;
		end = 600 * val(p[0]) + 60 * val(p[1]) + atoi(p + 2);
	}
	if (*tm == 0)
	{
		start = 0;
		end = DAYMIN - 1;
	}
	else
	{
		if (strlen(tm) < 4 || !isdigit((int) *tm))
			return 0;
		start = 600 * val(tm[0]) + 60 * val(tm[1]) + atoi(tm + 2);
		if (end < 0) end = start;
	}
	/* Treat 2400 as 0000, and do some more silent error checks. */
	if (end < 0) end = 0;
	if (start < 0) start = 0;
	if (end >= DAYMIN) end = DAYMIN - 1;
	if (start >= DAYMIN) start = DAYMIN - 1;

	timestr_debug("hour_fill: range from %d to %d\n", start, end);

	/*
	 *	Fill bitmap.
	 */
	i = start;
	while (1)
	{
		byte = (i >> 3);
		bit = i % 8;
		timestr_debug("setting byte %d, bit %d\n", byte, bit);
		bitmap[byte] |= (1 << bit);
		if (i == end) break;
		i++;
		i %= DAYMIN;
	}
	return 1;
}

/*
 *	Call the fill bitmap function for every day listed.
 */
static int Day_fill(char *bitmap, const char *tm, struct tm *stm, void *instance)
{
	SQL_INST *inst = instance;
	const char *hr;
	int n;
	int start, end;
	char buf[5];
	int hl_i;
	struct tm *tim;
	time_t ts;

	for (hr = tm; *hr; hr++)
		if (isdigit((int) *hr))
			break;
	if (hr == tm) 
		tm = "Al";

	timestr_debug("dayfill: hr %s    tm %s\n", hr, tm);

	while ((start = Strcode(&tm)) >= 0)
	{
		/*
		 *	Find start and end weekdays and
		 *	build a valid range 0 - 6.
		 */
		if (*tm == '-')
		{
			tm++;
			if ((end = Strcode(&tm)) < 0)
				break;
		}
		else
			end = start;
		if (start == 8)
		{
			tim = (struct tm *)rad_malloc(sizeof(struct tm));
        		memcpy(tim, stm, sizeof(struct tm));
			for (hl_i=0;hl_i<7;hl_i++)
			{
				ts = mktime(tim);
				localtime_r(&ts, tim);
				sprintf(buf,"%02d-%02d", tim->tm_mday, tim->tm_mon+1);
				if (nibs_is_holiday(buf, inst))
				{
					n = start = end = tim->tm_wday;
					timestr_debug("day_fill: range from %d to %d\n", start, end);
					while (1)
					{
						Hour_fill(bitmap + 180 * n, hr);
						if (n == end) break;
						n++;
						n %= 7;
					}
				}
				tim->tm_mday ++;
			}
			free(tim);
			break;
		}
		if (start == 7)
		{
			start = 1;
			end = 5;
		}
		if (start > 8)
		{
			start = 0;
			end = 6;
		}
		n = start;
		timestr_debug("day_fill: range from %d to %d\n", start, end);
		while (1)
		{
			Hour_fill(bitmap + 180 * n, hr);
			if (n == end) break;
			n++;
			n %= 7;
		}
	}
	return 1;
}

/*
 *	Fill the week bitmap with allowed times.
 */
static int Week_fill(char *bitmap, const char *tm, struct tm *stm, void *instance)
{
	SQL_INST *inst = instance;
	char *s;
	char tmp[128];

	strncpy(tmp, tm, 128);
	tmp[127] = 0;
	for (s = tmp; *s; s++)
		if (isupper(*s)) *s = tolower(*s);

	s = strtok(tmp, ",|");
	while (s)
	{
		Day_fill(bitmap, s, stm, inst);
		s = strtok(NULL, ",|");
	}

	return 0;
}

/*
 *	Match a timestring and return seconds left.
 *	-1 for no match, 0 for unlimited.
 */
int Timestr_match(const char *tmstr, time_t t, void *instance)
{
	SQL_INST *inst = instance;
	struct tm *tm, s_tm;
	char bitmap[WEEKMIN >> 3];
	int now, tot, i;
	int byte, bit;
#ifdef do_timestr_debug
	int y;
	char *s;
	char null[8];
#endif

	tm = localtime_r(&t, &s_tm);
	now = tm->tm_wday * DAYMIN + tm->tm_hour * 60 + tm->tm_min;
	tot = 0;
	memset(bitmap, 0, sizeof(bitmap));
	Week_fill(bitmap, tmstr, tm, inst);

#ifdef do_timestr_debug
	memset(null, 0, 8);
	for (i = 0; i < 7; i++)
	{
		timestr_debug("%d: ", i);
		s = bitmap + 180 * i;
		for (y = 0; y < 23; y++)
		{
			s = bitmap + 180 * i + (75 * y) / 10;
			timestr_debug("%c", memcmp(s, null, 8) == 0 ? '.' : '#');
		}
		timestr_debug("\n");
	}
#endif

	/*
	 *	See how many minutes we have.
	 */
	i = now;
	while (1)
	{
		byte = i >> 3;
		bit = i % 8;
		timestr_debug("READ: checking byte %d bit %d\n", byte, bit);
		if (!(bitmap[byte] & (1 << bit)))
			break;
		tot += 60;
		i++;
		i %= WEEKMIN;
		if (i == now)
			break;
	}

	if (tot == 0) 
		return -1;

	return (i == now) ? 0 : tot;
}

/*
 *  Smaller & faster than snprintf("%x");
 *  Completely stolen from ns_mta_md5 module
 */
void Pap_hexify(char *buffer, const char *str, int len)
{
	char ch;
	int i, j;
	
	for(i = 0; i < len; i ++)
	{
		ch = str[i];
		j = i << 1;
		buffer[j] = Pap_hextab[(ch>>4) & 15];
		buffer[j + 1] = Pap_hextab[ch & 15];
	}
	return;
}

/*************************************************************************
 *
 *	Function: strcmp_st
 *
 *	Purpose: Compare string whith pattern
 *               in pattern symbol '?' match any symbol in string,
 *               symbol '*' match any symbols to end of line.
 *
 *	Return:	 1 if string match pattern (with '*'),
 *               2 in case of full match (with ? also),
 *               0 elsewere
 *
 *************************************************************************/
int strcmp_st(const char * pattern, const char * str)
{
	size_t	size1, size2, i;

	size1 = strlen(str);
	size2 = strlen(pattern);

	for (i = 0; ((i < size1) && (i < size2)); i++) 
	{
		switch (pattern[i])
		{
			case '*':
				return 1;
			break;
			case '?':
				i++;
			break;
			default:
				if (str[i] != pattern[i])
					return 0;
			break;
		}
	}
	return ((i == size1) ? 2 : 0);
}


static char	dels[] = " ,;\t\n";

/*************************************************************************
 *
 *	Function: tokenize_st
 *
 *	Purpose: Extract word from string. Extracted word has been copyed
 *               in 'to' location.
 *
 *	Return:	 Pointer to next word in string,
 *               0 elsewere
 *
 *************************************************************************/
char * tokenize_st( char * str, char * to, const char * delim)
{
	char	* d;
	int	safe_count = 0;

	d = ( delim == (const char *)0) ? (char *)dels: (char *)delim;
	*to = (char)0;
	if ( str == 0)
		return 0;
	if ( *str == (char)0)
		return 0;
	while ( strchr( d, (int)*str) != 0)
	{
		str++;
		if ( *str == (char)0)
			return 0;
	}
	while ( strchr(d, (int)*str) == 0)
	{
		*to++ = *str++;
		safe_count ++;
		if ( (*str == (char)0) || (safe_count >= (ST_MAX_WORD_SIZE-1)))
			break;
	}
	*to = (char)0;
	return str;
}

/*************************************************************************
 *
 *	Function: find_word_st
 *
 *	Purpose: Find a word in string. String contain any word
 *               (may be with symbols '*', '?') with delimiters.
 *
 *	Return:	 1 if string match pattern (with '*'),
 *               2 in case of full match (with ? also),
 *               0 if word empty or NULL
 *               -1 if word not found
 *               -2 if string empty
 *               -3 if string is NULL
 *
 *************************************************************************/
int find_word_st( char * str, const char * word, const char * delim)
{
	char    buf[ST_MAX_WORD_SIZE];
	char	* ptr;
	int	res = -1;

	if ( str == 0)
		return -3;
	if ( *str == (char)0)
		return -2;
	if ( word == 0)
		return 0;
	if ( *word == (char)0)
		return 0;
	ptr = str;
	while ( (ptr = tokenize_st( ptr, buf, delim)) != 0)
	{
		res = strcmp_st( buf, word);
		if ( res)
			return res;
	}
	return -1;
}

int get_nas_type( uint32_t addr)
{
    NAS * nas;
    
    nas = nas_find( addr);
    if ( nas)
    {
	if ( (strncmp("cisco",nas->nastype,5) == 0) || (strncmp("CISCO",nas->nastype,5) == 0))
	    return NAS_CISCO;
	if ( (strncmp("ascend",nas->nastype,6) == 0) || (strncmp("ASCEND",nas->nastype,6) == 0))
	    return NAS_ASCEND;
    }
    
    return NAS_UNKNOW;
}
