#include <stdlib.h>
#include <string.h>
#include <time.h>

#include "radiusd.h"
#include "modules.h"
#include "conffile.h"
#include "rad_assert.h"
#include "rlm_nibs.h"
#include "nibs_utils.h"


/*************************************************************************
 *
 *	Function: nibs_find_time_cost
 *
 *	Purpose: Get cost by gid, week day and hour
 *
 *************************************************************************/
double nibs_find_time_cost(ULONGINT gid, int wday, int hour, void *instance)
{
	SQL_INST	*inst = instance;
	PRICES		*first;

	for (first = inst->config->first_price; first; first = first->next)
	{
		if ((first->gid == gid) && (first->week_day == wday))
		{
			switch (hour)
			{
				case 0: return first->h0; break;
				case 1: return first->h1; break;
				case 2: return first->h2; break;
				case 3: return first->h3; break;
				case 4: return first->h4; break;
				case 5: return first->h5; break;
				case 6: return first->h6; break;
				case 7: return first->h7; break;
				case 8: return first->h8; break;
				case 9: return first->h9; break;
				case 10: return first->h10; break;
				case 11: return first->h11; break;
				case 12: return first->h12; break;
				case 13: return first->h13; break;
				case 14: return first->h14; break;
				case 15: return first->h15; break;
				case 16: return first->h16; break;
				case 17: return first->h17; break;
				case 18: return first->h18; break;
				case 19: return first->h19; break;
				case 20: return first->h20; break;
				case 21: return first->h21; break;
				case 22: return first->h22; break;
				case 23: return first->h23; break;
				default: break;
			}
		}
	}
	return 0;
}

/*************************************************************************
 *
 *	Function: nibs_find_traffic_cost
 *
 *	Purpose: Get cost by gid, and direction
 *
 *************************************************************************/
double nibs_find_traffic_cost(ULONGINT gid, int wday, int hour, int direction, void *instance)
{
	SQL_INST	*inst = instance;
	PRICES	*first;

	for (first = inst->config->first_price; first; first = first->next)
	{
		if ((first->gid == gid) && (first->week_day == wday))
		{
			switch (direction)
			{
				case 1:
				switch (hour)
				{
					case 0: return first->input0; break;
					case 1: return first->input1; break;
					case 2: return first->input2; break;
					case 3: return first->input3; break;
					case 4: return first->input4; break;
					case 5: return first->input5; break;
					case 6: return first->input6; break;
					case 7: return first->input7; break;
					case 8: return first->input8; break;
					case 9: return first->input9; break;
				    	case 10: return first->input10; break;
					case 11: return first->input11; break;
					case 12: return first->input12; break;
					case 13: return first->input13; break;
					case 14: return first->input14; break;
					case 15: return first->input15; break;
					case 16: return first->input16; break;
					case 17: return first->input17; break;
					case 18: return first->input18; break;
					case 19: return first->input19; break;
					case 20: return first->input20; break;
					case 21: return first->input21; break;
					case 22: return first->input22; break;
					case 23: return first->input23; break;
					default: break;
				    }
				break;
				case 2:
				switch (hour)
				{
					case 0: return first->output0; break;
					case 1: return first->output1; break;
					case 2: return first->output2; break;
					case 3: return first->output3; break;
					case 4: return first->output4; break;
					case 5: return first->output5; break;
					case 6: return first->output6; break;
					case 7: return first->output7; break;
					case 8: return first->output8; break;
					case 9: return first->output9; break;
				    	case 10: return first->output10; break;
					case 11: return first->output11; break;
					case 12: return first->output12; break;
					case 13: return first->output13; break;
					case 14: return first->output14; break;
					case 15: return first->output15; break;
					case 16: return first->output16; break;
					case 17: return first->output17; break;
					case 18: return first->output18; break;
					case 19: return first->output19; break;
					case 20: return first->output20; break;
					case 21: return first->output21; break;
					case 22: return first->output22; break;
					case 23: return first->output23; break;
					default: break;
				}
				break;
				case 0:
					default: break;
			}
		}
	}
	return 0;
}

void cleanup_prices( void * instance)
{
        SQL_INST	*inst = instance;
	PRICES		*all_prices, *all_prices_next;

	/*Clear prices list*/
	for (all_prices = inst->config->first_price; all_prices; all_prices = all_prices_next)
	{
		all_prices_next = all_prices->next;
		free(all_prices);
	}
	inst->config->first_price = NULL;
	inst->config->num_prices = 0;
}

/*************************************************************************
 *
 *  Function: sql_init_prices
 *
 *  Purpose: Initialize prices
 *
 *************************************************************************/
int sql_init_prices(void *instance)
{
	SQLSOCK		*sqlsocket = NULL;
	SQL_INST	*inst = instance;
	SQL_ROW		row;
	char		querystr[MAX_QUERY_LEN];
	PRICES		*price;

/*Clear time_price list*/
	cleanup_prices( inst);
	memset(querystr, 0, MAX_QUERY_LEN);
/*Fill time_price*/

	if (inst->config->prices_select_query)
	{
		sqlsocket = als_get_socket(inst);
		if (sqlsocket == NULL)
			return RLM_MODULE_FAIL;
		query_log(NULL, inst, inst->config->prices_select_query);
		if (rlm_nibs_select_query(sqlsocket, inst, inst->config->prices_select_query))
		{
			snprintf( querystr, sizeof(querystr), "rlm_nibs (sql_init_prices): Couldn't get set of prices - %s",(inst->module->als_error)(sqlsocket, inst->config));
			return sql_mod_fail( inst, sqlsocket, querystr);
		}
		if ((inst->module->als_num_rows)(sqlsocket, inst->config) == 0)
			return sql_mod_ok( inst, sqlsocket, "rlm_nibs (sql_init_prices): No prices configured");
		while (rlm_nibs_fetch_row(sqlsocket, inst) == 0)
		{
			row = sqlsocket->row;
			if (!row)
				break;
			if ((price = rad_malloc(sizeof(PRICES))) == NULL)
				return sql_mod_fail( inst, sqlsocket, "rlm_nibs (sql_init_prices): out of memory");
			memset(price, 0, sizeof(PRICES));
			price->h0 = Atof(row[0]);
			price->input0 = Atof(row[1]);
			price->output0 = Atof(row[2]);
			price->h1 = Atof(row[3]);
			price->input1 = Atof(row[4]);
			price->output1 = Atof(row[5]);
			price->h2 = Atof(row[6]);
			price->input2 = Atof(row[7]);
			price->output2 = Atof(row[8]);
			price->h3 = Atof(row[9]);
			price->input3 = Atof(row[10]);
			price->output3 = Atof(row[11]);
			price->h4 = Atof(row[12]);
			price->input4 = Atof(row[13]);
			price->output4 = Atof(row[14]);
			price->h5 = Atof(row[15]);
			price->input5 = Atof(row[16]);
			price->output5 = Atof(row[17]);
			price->h6 = Atof(row[18]);
			price->input6 = Atof(row[19]);
			price->output6 = Atof(row[20]);
			price->h7 = Atof(row[21]);
			price->input7 = Atof(row[22]);
			price->output7 = Atof(row[23]);
			price->h8 = Atof(row[24]);
			price->input8 = Atof(row[25]);
			price->output8 = Atof(row[26]);
			price->h9 = Atof(row[27]);
			price->input9 = Atof(row[28]);
			price->output9 = Atof(row[29]);
			price->h10 = Atof(row[30]);
			price->input10 = Atof(row[31]);
			price->output10 = Atof(row[32]);
			price->h11 = Atof(row[33]);
			price->input11 = Atof(row[34]);
			price->output11 = Atof(row[35]);
			price->h12 = Atof(row[36]);
			price->input12 = Atof(row[37]);
			price->output12 = Atof(row[38]);
			price->h13 = Atof(row[39]);
			price->input13 = Atof(row[40]);
			price->output13 = Atof(row[41]);
			price->h14 = Atof(row[42]);
			price->input14 = Atof(row[43]);
			price->output14 = Atof(row[44]);
			price->h15 = Atof(row[45]);
			price->input15 = Atof(row[46]);
			price->output15 = Atof(row[47]);
			price->h16 = Atof(row[48]);
			price->input16 = Atof(row[49]);
			price->output16 = Atof(row[50]);
			price->h17 = Atof(row[51]);
			price->input17 = Atof(row[52]);
			price->output17 = Atof(row[53]);
			price->h18 = Atof(row[54]);
			price->input18 = Atof(row[55]);
			price->output18 = Atof(row[56]);
			price->h19 = Atof(row[57]);
			price->input19 = Atof(row[58]);
			price->output19 = Atof(row[59]);
			price->h20 = Atof(row[60]);
			price->input20 = Atof(row[61]);
			price->output20 = Atof(row[62]);
			price->h21 = Atof(row[63]);
			price->input21 = Atof(row[64]);
			price->output21 = Atof(row[65]);
			price->h22 = Atof(row[66]);
			price->input22 = Atof(row[67]);
			price->output22 = Atof(row[68]);
			price->h23 = Atof(row[69]);
			price->input23 = Atof(row[70]);
			price->output23 = Atof(row[71]);
			price->week_day = Atoi(row[72]);
			price->gid = Atoll(row[73]);

			price->next = inst->config->first_price;
			inst->config->first_price = price;
			inst->config->num_prices++;
		}
		sql_mod_ok(inst, sqlsocket, NULL);
	}
	return RLM_MODULE_OK;
}
