/***************************************************************************
*  rlm_nibs.h                          rlm_nibs - FreeRADIUS SQL Module    *
*                                                                          *
*      Header for main SQL module file                                     *
*                                                                          *
*                               Neon aka Sergey Derkach <neon@is.kh.ua>    *
*                                     Mike Machado <mike@innercite.com>    *
***************************************************************************/
#ifndef _rlm_nibs_H
#define _rlm_nibs_H

#ifdef HAVE_PTHREAD_H
#include        <pthread.h>
#endif

#include	<ltdl.h>

#include "conf.h"
#include "conffile.h"

#define SQLSOCK_LOCKED		0
#define SQLSOCK_UNLOCKED	1

#define PW_VP_USERDATA		1
#define PW_VP_GROUPDATA		2
#define PW_VP_REALMDATA		3

#define PW_ITEM_CHECK			0
#define PW_ITEM_REPLY			1

typedef char** SQL_ROW;

typedef struct sql_socket {
	int     id;
#ifdef HAVE_PTHREAD_H
	pthread_mutex_t mutex;
#endif
	struct sql_socket *next;
	enum { sockconnected, sockunconnected } state;

	void	*conn;
	SQL_ROW row;
} SQLSOCK;

typedef struct rlm_nibs_module_t {
	const char *name;
	int (*als_init_socket)(SQLSOCK *sqlsocket, NIBS_CONFIG *config);
	int (*als_destroy_socket)(SQLSOCK *sqlsocket, NIBS_CONFIG *config);
	int (*als_query)(SQLSOCK *sqlsocket, NIBS_CONFIG *config, const char *query);
	int (*als_select_query)(SQLSOCK *sqlsocket, NIBS_CONFIG *config, const char *query);
	int (*als_store_result)(SQLSOCK *sqlsocket, NIBS_CONFIG *config);
	int (*als_num_fields)(SQLSOCK *sqlsocket, NIBS_CONFIG *config);
	int (*als_num_rows)(SQLSOCK *sqlsocket, NIBS_CONFIG *config);
	int (*als_fetch_row)(SQLSOCK *sqlsocket, NIBS_CONFIG *config);
	int (*als_free_result)(SQLSOCK *sqlsocket, NIBS_CONFIG *config);
	const char *(*als_error)(SQLSOCK *sqlsocket, NIBS_CONFIG *config);
	int (*als_close)(SQLSOCK *sqlsocket, NIBS_CONFIG *config);
	int (*als_finish_query)(SQLSOCK *sqlsocket, NIBS_CONFIG *config);
	int (*als_finish_select_query)(SQLSOCK *sqlsocket, NIBS_CONFIG *config);
	int (*als_affected_rows)(SQLSOCK *sqlsocket, NIBS_CONFIG *config);
} rlm_nibs_module_t;

typedef struct sql_inst {
	time_t		connect_after;
	SQLSOCK		*sqlpool;
	SQLSOCK		*last_used;
	NIBS_CONFIG	*config;

	lt_dlhandle handle;
	rlm_nibs_module_t *module;
} SQL_INST;


int     als_init_socketpool(SQL_INST * inst);
void    als_poolfree(SQL_INST * inst);
int     als_close_socket(SQL_INST *inst, SQLSOCK * sqlsocket);
SQLSOCK *als_get_socket(SQL_INST * inst);
int     als_release_socket(SQL_INST * inst, SQLSOCK * sqlsocket);
void    query_log(REQUEST *request, SQL_INST * inst, const char *querystr);
int	rlm_nibs_select_query(SQLSOCK *sqlsocket, SQL_INST *inst, const char *query);
int	rlm_nibs_query(SQLSOCK *sqlsocket, SQL_INST *inst, const char *query);
int	rlm_nibs_fetch_row(SQLSOCK *sqlsocket, SQL_INST *inst);
int     als_escape_func(char *out, int outlen, const char *in);
void mod_cleanup( REQUEST * request, NIBS_USER * nibs_user);
int mod_fail( REQUEST * request, NIBS_USER * nibs_user, const char * str);
int mod_ok( REQUEST * request, NIBS_USER * nibs_user);
int sql_mod_fail( SQL_INST * inst, SQLSOCK * sqlsocket, const char * str);
int sql_mod_ok( SQL_INST * inst, SQLSOCK * sqlsocket, const char * str);
int as_bool( int c);

#endif
